#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import uuid
import unittest
import utils
from utils import get_default_trace_class
from bt2 import trace_class as bt2_trace_class
from bt2 import value as bt2_value
from bt2 import trace as bt2_trace
from bt2 import stream as bt2_stream
from bt2 import utils as bt2_utils


class TraceTestCase(unittest.TestCase):
    def setUp(self):
        self._tc = get_default_trace_class()

    def test_create_default(self):
        trace = self._tc()
        self.assertIsNone(trace.name)
        self.assertIsNone(trace.uuid)
        self.assertEqual(len(trace.environment), 0)
        self.assertEqual(len(trace.user_attributes), 0)

    def test_create_invalid_name(self):
        with self.assertRaises(TypeError):
            self._tc(name=17)

    def test_create_user_attributes(self):
        trace = self._tc(user_attributes={'salut': 23})
        self.assertEqual(trace.user_attributes, {'salut': 23})
        self.assertIs(type(trace.user_attributes), bt2_value.MapValue)

    def test_create_invalid_user_attributes(self):
        with self.assertRaises(TypeError):
            self._tc(user_attributes=object())

    def test_create_invalid_user_attributes_value_type(self):
        with self.assertRaises(TypeError):
            self._tc(user_attributes=23)

    def test_attr_trace_class(self):
        trace = self._tc()
        self.assertEqual(trace.cls.addr, self._tc.addr)
        self.assertIs(type(trace.cls), bt2_trace_class._TraceClass)

    def test_const_attr_trace_class(self):
        trace = utils.get_const_stream_beginning_message().stream.trace
        self.assertIs(type(trace.cls), bt2_trace_class._TraceClassConst)

    def test_attr_name(self):
        trace = self._tc(name='mein trace')
        self.assertEqual(trace.name, 'mein trace')

    def test_attr_uuid(self):
        trace = self._tc(uuid=uuid.UUID('da7d6b6f-3108-4706-89bd-ab554732611b'))
        self.assertEqual(trace.uuid, uuid.UUID('da7d6b6f-3108-4706-89bd-ab554732611b'))

    def test_env_get(self):
        trace = self._tc(environment={'hello': 'you', 'foo': -5})
        self.assertIs(type(trace.environment), bt2_trace._TraceEnvironment)
        self.assertIs(type(trace.environment['foo']), bt2_value.SignedIntegerValue)
        self.assertEqual(trace.environment['hello'], 'you')
        self.assertEqual(trace.environment['foo'], -5)

    def test_env_iter(self):
        trace = self._tc(environment={'hello': 'you', 'foo': -5})
        values = set(trace.environment)
        self.assertEqual(values, {'hello', 'foo'})

    def test_const_env_get(self):
        trace = utils.get_const_stream_beginning_message().stream.trace
        self.assertIs(type(trace.environment), bt2_trace._TraceEnvironmentConst)
        self.assertIs(
            type(trace.environment['patate']), bt2_value._SignedIntegerValueConst
        )

    def test_const_env_iter(self):
        trace = utils.get_const_stream_beginning_message().stream.trace
        values = set(trace.environment)
        self.assertEqual(values, {'patate'})

    def test_const_env_set(self):
        trace = utils.get_const_stream_beginning_message().stream.trace
        with self.assertRaises(TypeError):
            trace.environment['patate'] = 33

    def test_env_get_non_existent(self):
        trace = self._tc(environment={'hello': 'you', 'foo': -5})

        with self.assertRaises(KeyError):
            trace.environment['lel']

    def test_len(self):
        trace = self._tc()
        sc = self._tc.create_stream_class()
        self.assertEqual(len(trace), 0)

        trace.create_stream(sc)
        self.assertEqual(len(trace), 1)

    def _create_trace_with_some_streams(self):
        sc = self._tc.create_stream_class(assigns_automatic_stream_id=False)
        trace = self._tc()
        trace.create_stream(sc, id=12)
        trace.create_stream(sc, id=15)
        trace.create_stream(sc, id=17)

        return trace

    def test_iter(self):
        trace = self._create_trace_with_some_streams()
        stream_ids = set(trace)
        self.assertEqual(stream_ids, {12, 15, 17})

    def test_getitem(self):
        trace = self._create_trace_with_some_streams()
        self.assertEqual(trace[12].id, 12)
        self.assertIs(type(trace[12]), bt2_stream._Stream)

    def test_const_getitem(self):
        trace = utils.get_const_stream_beginning_message().stream.trace
        self.assertIs(type(trace[0]), bt2_stream._StreamConst)

    def test_getitem_invalid_key(self):
        trace = self._create_trace_with_some_streams()
        with self.assertRaises(KeyError):
            trace[18]

    def test_destruction_listener(self):
        def on_trace_class_destruction(trace_class):
            nonlocal num_trace_class_destroyed_calls
            num_trace_class_destroyed_calls += 1

        def on_trace_destruction(trace):
            nonlocal num_trace_destroyed_calls
            num_trace_destroyed_calls += 1

        num_trace_class_destroyed_calls = 0
        num_trace_destroyed_calls = 0

        trace_class = get_default_trace_class()
        stream_class = trace_class.create_stream_class()
        trace = trace_class()
        stream = trace.create_stream(stream_class)

        trace_class.add_destruction_listener(on_trace_class_destruction)
        td_handle1 = trace.add_destruction_listener(on_trace_destruction)
        td_handle2 = trace.add_destruction_listener(on_trace_destruction)

        self.assertIs(type(td_handle1), bt2_utils._ListenerHandle)

        trace.remove_destruction_listener(td_handle2)

        self.assertEqual(num_trace_class_destroyed_calls, 0)
        self.assertEqual(num_trace_destroyed_calls, 0)

        del trace

        self.assertEqual(num_trace_class_destroyed_calls, 0)
        self.assertEqual(num_trace_destroyed_calls, 0)

        del stream

        self.assertEqual(num_trace_class_destroyed_calls, 0)
        self.assertEqual(num_trace_destroyed_calls, 1)

        del trace_class

        self.assertEqual(num_trace_class_destroyed_calls, 0)
        self.assertEqual(num_trace_destroyed_calls, 1)

        del stream_class

        self.assertEqual(num_trace_class_destroyed_calls, 1)
        self.assertEqual(num_trace_destroyed_calls, 1)

    def test_remove_destruction_listener_wrong_type(self):
        trace_class = get_default_trace_class()
        trace = trace_class()

        with self.assertRaisesRegex(
            TypeError, r"'int' is not a '<class 'bt2.utils._ListenerHandle'>' object"
        ):
            trace.remove_destruction_listener(123)

    def test_remove_destruction_listener_wrong_object(self):
        def on_trace_destruction(trace):
            pass

        trace_class_1 = get_default_trace_class()
        trace1 = trace_class_1()
        trace_class_2 = get_default_trace_class()
        trace2 = trace_class_2()

        handle1 = trace1.add_destruction_listener(on_trace_destruction)

        with self.assertRaisesRegex(
            ValueError,
            r'This trace destruction listener does not match the trace object\.',
        ):
            trace2.remove_destruction_listener(handle1)

    def test_remove_destruction_listener_twice(self):
        def on_trace_destruction(trace_class):
            pass

        trace_class = get_default_trace_class()
        trace = trace_class()
        handle = trace.add_destruction_listener(on_trace_destruction)

        trace.remove_destruction_listener(handle)

        with self.assertRaisesRegex(
            ValueError, r'This trace destruction listener was already removed\.'
        ):
            trace.remove_destruction_listener(handle)

    def test_raise_in_destruction_listener(self):
        def on_trace_destruction(trace):
            raise ValueError('it hurts')

        trace_class = get_default_trace_class()
        trace = trace_class()
        trace.add_destruction_listener(on_trace_destruction)

        del trace


if __name__ == '__main__':
    unittest.main()



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import bt2
import unittest


class PackageTestCase(unittest.TestCase):
    def _assert_in_bt2(self, name):
        self.assertIn(name, dir(bt2))


_public_names = [
    '__version__',
    '_ArrayField',
    '_ArrayFieldClass',
    '_ArrayFieldClassConst',
    '_ArrayFieldConst',
    '_ArrayValueConst',
    '_BitArrayField',
    '_BitArrayFieldClass',
    '_BitArrayFieldClassConst',
    '_BitArrayFieldConst',
    '_BoolField',
    '_BoolFieldClass',
    '_BoolFieldClassConst',
    '_BoolFieldConst',
    '_BoolValueConst',
    '_ClockSnapshotConst',
    '_ComponentClassErrorCause',
    '_ComponentErrorCause',
    '_CurrentArrayElementFieldPathItem',
    '_CurrentOptionContentFieldPathItem',
    '_DiscardedEventsMessage',
    '_DiscardedEventsMessageConst',
    '_DiscardedPacketsMessage',
    '_DiscardedPacketsMessageConst',
    '_DoublePrecisionRealField',
    '_DoublePrecisionRealFieldConst',
    '_DynamicArrayField',
    '_DynamicArrayFieldClass',
    '_DynamicArrayFieldClassConst',
    '_DynamicArrayFieldConst',
    '_DynamicArrayWithLengthFieldFieldClass',
    '_DynamicArrayWithLengthFieldFieldClassConst',
    '_EnumerationField',
    '_EnumerationFieldClass',
    '_EnumerationFieldClassConst',
    '_EnumerationFieldConst',
    '_Error',
    '_ErrorCause',
    '_EventMessage',
    '_EventMessageConst',
    '_FilterComponentClassConst',
    '_FilterComponentConst',
    '_IndexFieldPathItem',
    '_IntegerField',
    '_IntegerFieldClass',
    '_IntegerFieldClassConst',
    '_IntegerFieldConst',
    '_IntegerValue',
    '_IntegerValueConst',
    '_MapValueConst',
    '_MessageIteratorErrorCause',
    '_MessageIteratorInactivityMessage',
    '_MessageIteratorInactivityMessageConst',
    '_OptionFieldClass',
    '_OptionFieldClassConst',
    '_OptionWithBoolSelectorFieldClass',
    '_OptionWithBoolSelectorFieldClassConst',
    '_OptionWithIntegerSelectorFieldClass',
    '_OptionWithIntegerSelectorFieldClassConst',
    '_OptionWithSelectorFieldClass',
    '_OptionWithSelectorFieldClassConst',
    '_OptionWithSignedIntegerSelectorFieldClass',
    '_OptionWithSignedIntegerSelectorFieldClassConst',
    '_OptionWithUnsignedIntegerSelectorFieldClass',
    '_OptionWithUnsignedIntegerSelectorFieldClassConst',
    '_PacketBeginningMessage',
    '_PacketBeginningMessageConst',
    '_PacketEndMessage',
    '_PacketEndMessageConst',
    '_RealField',
    '_RealFieldClass',
    '_RealFieldClassConst',
    '_RealFieldConst',
    '_RealValueConst',
    '_SignedEnumerationField',
    '_SignedEnumerationFieldClass',
    '_SignedEnumerationFieldClassConst',
    '_SignedEnumerationFieldConst',
    '_SignedIntegerField',
    '_SignedIntegerFieldClass',
    '_SignedIntegerFieldClassConst',
    '_SignedIntegerFieldConst',
    '_SignedIntegerRangeConst',
    '_SignedIntegerRangeSetConst',
    '_SignedIntegerValueConst',
    '_SinglePrecisionRealField',
    '_SinglePrecisionRealFieldConst',
    '_SinkComponentClassConst',
    '_SinkComponentConst',
    '_SourceComponentClassConst',
    '_SourceComponentConst',
    '_StaticArrayField',
    '_StaticArrayFieldClass',
    '_StaticArrayFieldClassConst',
    '_StaticArrayFieldConst',
    '_StreamBeginningMessage',
    '_StreamBeginningMessageConst',
    '_StreamEndMessage',
    '_StreamEndMessageConst',
    '_StringField',
    '_StringFieldClass',
    '_StringFieldClassConst',
    '_StringFieldConst',
    '_StringValueConst',
    '_StructureField',
    '_StructureFieldClass',
    '_StructureFieldClassConst',
    '_StructureFieldConst',
    '_UnknownClockSnapshot',
    '_UnsignedEnumerationField',
    '_UnsignedEnumerationFieldClass',
    '_UnsignedEnumerationFieldClassConst',
    '_UnsignedEnumerationFieldConst',
    '_UnsignedIntegerField',
    '_UnsignedIntegerFieldClass',
    '_UnsignedIntegerFieldClassConst',
    '_UnsignedIntegerFieldConst',
    '_UnsignedIntegerRangeConst',
    '_UnsignedIntegerRangeSetConst',
    '_UnsignedIntegerValueConst',
    '_UserFilterComponent',
    '_UserMessageIterator',
    '_UserSinkComponent',
    '_UserSourceComponent',
    '_VariantField',
    '_VariantField',
    '_VariantFieldClass',
    '_VariantFieldClassConst',
    '_VariantFieldClassWithIntegerSelector',
    '_VariantFieldClassWithIntegerSelectorConst',
    '_VariantFieldClassWithoutSelector',
    '_VariantFieldClassWithoutSelectorConst',
    '_VariantFieldClassWithSignedIntegerSelector',
    '_VariantFieldClassWithSignedIntegerSelectorConst',
    '_VariantFieldClassWithUnsignedIntegerSelector',
    '_VariantFieldClassWithUnsignedIntegerSelectorConst',
    '_VariantFieldConst',
    '_VariantFieldConst',
    'ArrayValue',
    'AutoSourceComponentSpec',
    'BoolValue',
    'ClockClassOffset',
    'ComponentClassType',
    'ComponentDescriptor',
    'ComponentSpec',
    'create_value',
    'EventClassLogLevel',
    'FieldPathScope',
    'find_plugin',
    'find_plugins',
    'find_plugins_in_path',
    'get_global_logging_level',
    'get_minimal_logging_level',
    'Graph',
    'IntegerDisplayBase',
    'Interrupter',
    'LoggingLevel',
    'MapValue',
    'plugin_component_class',
    'QueryExecutor',
    'RealValue',
    'register_plugin',
    'set_global_logging_level',
    'SignedIntegerRange',
    'SignedIntegerRangeSet',
    'SignedIntegerValue',
    'StringValue',
    'TraceCollectionMessageIterator',
    'UnsignedIntegerRange',
    'UnsignedIntegerRangeSet',
    'UnsignedIntegerValue',
]

for name in _public_names:

    def test_func(self):
        self._assert_in_bt2(name)

    setattr(PackageTestCase, 'test_has_' + name, test_func)


if __name__ == '__main__':
    unittest.main()



import bt2


class TheIteratorOfConfusion(bt2._UserMessageIterator):
    def __init__(self, config, port):
        self._at = 0
        test_name = port.user_data[0]
        TEST_CASES[test_name].create_msgs(self, port.user_data[1:])

    def __next__(self):
        if self._at < len(self._msgs):
            msg = self._msgs[self._at]
            self._at += 1
            return msg
        raise StopIteration


@bt2.plugin_component_class
class TheSourceOfConfusion(
    bt2._UserSourceComponent, message_iterator_class=TheIteratorOfConfusion
):
    def __init__(self, config, params, obj):
        test_name = str(params['test-name'])

        TEST_CASES[test_name].source_setup(self, test_name)


class DiffTraceName:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class()
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class()
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        trace_name1 = 'rouyn'
        trace_name2 = 'noranda'

        src._add_output_port('out1', (test_name, 1, tc1, cc1, trace_name1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, trace_name2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, trace_name = params
        trace = tc(name=trace_name)
        sc = tc.create_stream_class(
            default_clock_class=cc, assigns_automatic_stream_id=False
        )
        stream = trace.create_stream(sc, 0)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class DiffStreamName:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class()
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class()
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        stream_name1 = 'port-daniel'
        stream_name2 = 'gascon'

        src._add_output_port('out1', (test_name, 1, tc1, cc1, stream_name1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, stream_name2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, stream_name = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc, assigns_automatic_stream_id=False
        )
        stream = trace.create_stream(sc, 0, stream_name)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class DiffStreamId:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class()
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class()
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        stream_id1 = 18
        stream_id2 = 23

        src._add_output_port('out1', (test_name, 1, tc1, cc1, stream_id1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, stream_id2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, stream_id = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc, assigns_automatic_stream_id=False
        )
        stream = trace.create_stream(sc, stream_id)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class DiffStreamNoName:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class()
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class()
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        stream_name1 = "one"
        stream_name2 = None

        src._add_output_port('out1', (test_name, 1, tc1, cc1, stream_name1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, stream_name2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, stream_no_name = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc, assigns_automatic_stream_id=False
        )
        stream = trace.create_stream(sc, 0, name=stream_no_name)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class DiffStreamClassId:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        stream_class_id1 = 18
        stream_class_id2 = 23

        src._add_output_port('out1', (test_name, 1, tc1, cc1, stream_class_id1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, stream_class_id2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, stream_class_id = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc,
            id=stream_class_id,
            assigns_automatic_stream_id=False,
        )
        stream = trace.create_stream(sc, 0)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class DiffStreamClassName:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        stream_class_name1 = 'one'
        stream_class_name2 = 'two'

        src._add_output_port('out1', (test_name, 1, tc1, cc1, stream_class_name1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, stream_class_name2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, stream_class_name = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc,
            id=0,
            name=stream_class_name,
            assigns_automatic_stream_id=False,
        )
        stream = trace.create_stream(sc, 0)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class DiffStreamClassNoName:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        stream_class_name1 = 'one'
        stream_class_name2 = None

        src._add_output_port('out1', (test_name, 1, tc1, cc1, stream_class_name1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, stream_class_name2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, stream_class_name = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc,
            id=0,
            name=stream_class_name,
            assigns_automatic_stream_id=False,
        )
        stream = trace.create_stream(sc, 0)

        sb_msg = msg_iter._create_stream_beginning_message(stream, 0)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class BasicTimestampOrdering:
    def source_setup(src, test_name):
        tc = src._create_trace_class()
        cc = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        timestamp1 = 0
        timestamp2 = 120
        timestamp3 = 4

        src._add_output_port('out1', (test_name, 1, tc, cc, timestamp1))
        src._add_output_port('out2', (test_name, 2, tc, cc, timestamp2))
        src._add_output_port('out3', (test_name, 3, tc, cc, timestamp3))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, timestamp = params
        trace = tc()
        sc = tc.create_stream_class(default_clock_class=cc)
        stream = trace.create_stream(sc)

        sb_msg = msg_iter._create_stream_beginning_message(stream, timestamp)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, se_msg]


class MultiIterOrdering:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        tc2 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        tc3 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        tc4 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        src._add_output_port('out1', (test_name, 1, tc1, cc))
        src._add_output_port('out2', (test_name, 2, tc2, cc))
        src._add_output_port('out3', (test_name, 3, tc3, cc))
        src._add_output_port('out4', (test_name, 4, tc4, cc))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc = params
        trace_hello = tc(name='hello')
        trace_meow = tc(name='meow')

        # Craft list of messages for each iterator so that the last messages of
        # each iterator are all sharing the same timestamp.
        clock_snapshot_value = 25

        if iter_id == 1:
            # Event, 2500 ns, trace "hello", stream class 0, stream 1
            stream_class0 = tc.create_stream_class(
                id=0, default_clock_class=cc, assigns_automatic_stream_id=False
            )
            sc_0_stream_1 = trace_hello.create_stream(stream_class0, id=1)
            event_class = stream_class0.create_event_class(name='saumon atlantique')

            msg_iter._msgs = [
                msg_iter._create_stream_beginning_message(sc_0_stream_1, 0),
                msg_iter._create_event_message(
                    event_class, sc_0_stream_1, clock_snapshot_value
                ),
                msg_iter._create_stream_end_message(sc_0_stream_1, iter_id * 193),
            ]
        elif iter_id == 2:
            # Packet beginning, 2500 ns, trace "meow", stream class 0, stream 1
            stream_class0 = tc.create_stream_class(
                id=0,
                default_clock_class=cc,
                supports_packets=True,
                packets_have_beginning_default_clock_snapshot=True,
                packets_have_end_default_clock_snapshot=True,
                assigns_automatic_stream_id=False,
            )

            sc_0_stream_1 = trace_meow.create_stream(stream_class0, id=1)
            packet = sc_0_stream_1.create_packet()

            msg_iter._msgs = [
                msg_iter._create_stream_beginning_message(sc_0_stream_1, 1),
                msg_iter._create_packet_beginning_message(packet, clock_snapshot_value),
                msg_iter._create_packet_end_message(packet, iter_id * 79),
                msg_iter._create_stream_end_message(sc_0_stream_1, iter_id * 193),
            ]
        elif iter_id == 3:
            # Stream beginning, 2500 ns, trace "hello", stream class 0, stream 0
            stream_class0 = tc.create_stream_class(
                id=0, default_clock_class=cc, assigns_automatic_stream_id=False
            )

            sc_0_stream_0 = trace_hello.create_stream(stream_class0, id=0)

            msg_iter._msgs = [
                msg_iter._create_stream_beginning_message(
                    sc_0_stream_0, clock_snapshot_value
                ),
                msg_iter._create_stream_end_message(sc_0_stream_0, iter_id * 193),
            ]
        elif iter_id == 4:
            # Event, 2500 ns, trace "meow", stream class 1, stream 1
            stream_class1 = tc.create_stream_class(
                id=1, default_clock_class=cc, assigns_automatic_stream_id=False
            )

            sc_1_stream_1 = trace_meow.create_stream(stream_class1, id=1)

            event_class = stream_class1.create_event_class(name='bar rayé')
            msg_iter._msgs = [
                msg_iter._create_stream_beginning_message(sc_1_stream_1, 3),
                msg_iter._create_event_message(
                    event_class, sc_1_stream_1, clock_snapshot_value
                ),
                msg_iter._create_stream_end_message(sc_1_stream_1, iter_id * 193),
            ]


class DiffEventClassName:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        event_class_name1 = 'Hull'
        event_class_name2 = 'Gatineau'

        src._add_output_port('out1', (test_name, 1, tc1, cc1, event_class_name1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, event_class_name2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, event_class_name = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc,
            id=0,
            assigns_automatic_stream_id=False,
            supports_packets=False,
        )
        ec = sc.create_event_class(name=event_class_name)

        stream = trace.create_stream(sc, 0)

        # Use event class name length as timestamp so that both stream
        # beginning message are not at the same time. This test is targetting
        # event message.
        sb_msg = msg_iter._create_stream_beginning_message(stream, len(ec.name))
        ev_msg = msg_iter._create_event_message(ec, stream, 50)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, ev_msg, se_msg]


class DiffEventClassId:
    def source_setup(src, test_name):
        tc1 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc1 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))
        tc2 = src._create_trace_class(assigns_automatic_stream_class_id=False)
        cc2 = src._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(0))

        event_class_id1 = 1
        event_class_id2 = 2

        src._add_output_port('out1', (test_name, 1, tc1, cc1, event_class_id1))
        src._add_output_port('out2', (test_name, 2, tc2, cc2, event_class_id2))

    def create_msgs(msg_iter, params):
        iter_id, tc, cc, event_class_id = params
        trace = tc()
        sc = tc.create_stream_class(
            default_clock_class=cc,
            id=0,
            assigns_automatic_stream_id=False,
            assigns_automatic_event_class_id=False,
            supports_packets=False,
        )
        ec = sc.create_event_class(id=event_class_id)

        stream = trace.create_stream(sc, 0)

        # Use event class id as timestamp so that both stream beginning message
        # are not at the same time. This test is targetting event message.
        sb_msg = msg_iter._create_stream_beginning_message(stream, ec.id)
        ev_msg = msg_iter._create_event_message(ec, stream, 50)
        se_msg = msg_iter._create_stream_end_message(stream, iter_id * 193)

        msg_iter._msgs = [sb_msg, ev_msg, se_msg]


class DiffInactivityMsgCs:
    def source_setup(src, test_name):
        cc1 = src._create_clock_class(
            frequency=1, name='La Baie', offset=bt2.ClockClassOffset(0)
        )
        cc2 = src._create_clock_class(
            frequency=1, name='Chicoutimi', offset=bt2.ClockClassOffset(0)
        )

        src._add_output_port('out1', (test_name, cc1))
        src._add_output_port('out2', (test_name, cc2))

    def create_msgs(msg_iter, params):
        (cc,) = params
        sb_msg = msg_iter._create_message_iterator_inactivity_message(cc, 0)
        msg_iter._msgs = [sb_msg]


TEST_CASES = {
    'diff_trace_name': DiffTraceName,
    'diff_event_class_name': DiffEventClassName,
    'diff_event_class_id': DiffEventClassId,
    'diff_stream_name': DiffStreamName,
    'diff_stream_no_name': DiffStreamNoName,
    'diff_stream_id': DiffStreamId,
    'diff_stream_class_id': DiffStreamClassId,
    'diff_stream_class_name': DiffStreamClassName,
    'diff_stream_class_no_name': DiffStreamClassNoName,
    'diff_inactivity_msg_cs': DiffInactivityMsgCs,
    'basic_timestamp_ordering': BasicTimestampOrdering,
    'multi_iter_ordering': MultiIterOrdering,
}

bt2.register_plugin(__name__, 'test-muxer')



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import bt2
import unittest


class InterrupterTestCase(unittest.TestCase):
    def setUp(self):
        self._interrupter = bt2.Interrupter()

    def test_create(self):
        self.assertFalse(self._interrupter.is_set)

    def test_is_set(self):
        self.assertFalse(self._interrupter.is_set)

    def test_bool(self):
        self.assertFalse(self._interrupter)
        self._interrupter.set()
        self.assertTrue(self._interrupter)

    def test_set(self):
        self.assertFalse(self._interrupter)
        self._interrupter.set()
        self.assertTrue(self._interrupter)

    def test_reset(self):
        self._interrupter.set()
        self.assertTrue(self._interrupter)
        self._interrupter.reset()
        self.assertFalse(self._interrupter)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2016, Matt Layman

import os
from unittest import TextTestResult, TextTestRunner
from unittest.runner import _WritelnDecorator
import sys

from tap import formatter
from tap.i18n import _
from tap.tracker import Tracker


class TAPTestResult(TextTestResult):

    FORMAT = None

    def __init__(self, stream, descriptions, verbosity):
        super(TAPTestResult, self).__init__(stream, descriptions, verbosity)

    def stopTestRun(self):
        """Once the test run is complete, generate each of the TAP files."""
        super(TAPTestResult, self).stopTestRun()
        self.tracker.generate_tap_reports()

    def addError(self, test, err):
        super(TAPTestResult, self).addError(test, err)
        diagnostics = formatter.format_exception(err)
        self.tracker.add_not_ok(
            self._cls_name(test), self._description(test),
            diagnostics=diagnostics)

    def addFailure(self, test, err):
        super(TAPTestResult, self).addFailure(test, err)
        diagnostics = formatter.format_exception(err)
        self.tracker.add_not_ok(
            self._cls_name(test), self._description(test),
            diagnostics=diagnostics)

    def addSuccess(self, test):
        super(TAPTestResult, self).addSuccess(test)
        self.tracker.add_ok(self._cls_name(test), self._description(test))

    def addSkip(self, test, reason):
        super(TAPTestResult, self).addSkip(test, reason)
        self.tracker.add_skip(
            self._cls_name(test), self._description(test), reason)

    def addExpectedFailure(self, test, err):
        super(TAPTestResult, self).addExpectedFailure(test, err)
        diagnostics = formatter.format_exception(err)
        self.tracker.add_not_ok(
            self._cls_name(test), self._description(test),
            _('(expected failure)'), diagnostics=diagnostics)

    def addUnexpectedSuccess(self, test):
        super(TAPTestResult, self).addUnexpectedSuccess(test)
        self.tracker.add_ok(self._cls_name(test), self._description(test),
                            _('(unexpected success)'))

    def _cls_name(self, test):
        return test.__class__.__name__

    def _description(self, test):
        if self.FORMAT:
            try:
                return self.FORMAT.format(
                    method_name=str(test),
                    short_description=test.shortDescription() or '')
            except KeyError:
                sys.exit(_(
                    'Bad format string: {format}\n'
                    'Replacement options are: {{short_description}} and '
                    '{{method_name}}').format(format=self.FORMAT))

        return test.shortDescription() or str(test)


# TODO: 2016-7-30 mblayman - Since the 2.6 signature is no longer relevant,
# check the possibility of removing the module level scope.

# Module level state stinks, but this is the only way to keep compatibility
# with Python 2.6. The best place for the tracker is as an instance variable
# on the runner, but __init__ is so different that it is not easy to create
# a runner that satisfies every supported Python version.
_tracker = Tracker()


class TAPTestRunner(TextTestRunner):
    """A test runner that will behave exactly like TextTestRunner and will
    additionally generate TAP files for each test case"""

    resultclass = TAPTestResult

    def set_stream(self, streaming):
        """Set the streaming boolean option to stream TAP directly to stdout.

        The test runner default output will be suppressed in favor of TAP.
        """
        self.stream = _WritelnDecorator(open(os.devnull, 'w'))
        _tracker.streaming = streaming
        _tracker.stream = sys.stdout

    def _makeResult(self):
        result = self.resultclass(
            self.stream, self.descriptions, self.verbosity)
        result.tracker = _tracker
        return result

    @classmethod
    def set_outdir(cls, outdir):
        """Set the output directory so that TAP files are written to the
        specified outdir location.
        """
        # Blame the lack of unittest extensibility for this hacky method.
        _tracker.outdir = outdir

    @classmethod
    def set_combined(cls, combined):
        """Set the tracker to use a single output file."""
        _tracker.combined = combined

    @classmethod
    def set_header(cls, header):
        """Set the header display flag."""
        _tracker.header = header

    @classmethod
    def set_format(cls, fmt):
        """Set the format of each test line.

        The format string can use:
        * {method_name}: The test method name
        * {short_description}: The test's docstring short description
        """
        TAPTestResult.FORMAT = fmt



# Copyright (c) 2016, Matt Layman

import gettext
import os

localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
translate = gettext.translation('tappy', localedir, fallback=True)
_ = translate.gettext



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import unittest
import bt2
import utils
from utils import TestOutputPortMessageIterator
from bt2 import clock_snapshot as bt2_clock_snapshot
from bt2 import event as bt2_event
from bt2 import event_class as bt2_event_class
from bt2 import field as bt2_field
from bt2 import packet as bt2_packet
from bt2 import stream as bt2_stream
from bt2 import stream_class as bt2_stream_class
from bt2 import trace as bt2_trace
from bt2 import trace_class as bt2_trace_class


class AllMessagesTestCase(unittest.TestCase):
    def setUp(self):
        class MyIter(bt2._UserMessageIterator):
            def __init__(self, config, self_port_output):
                self._at = 0
                self._with_stream_msgs_clock_snapshots = self_port_output.user_data.get(
                    'with_stream_msgs_clock_snapshots', False
                )

            def __next__(self):
                if test_obj._clock_class:
                    if self._at == 0:
                        if self._with_stream_msgs_clock_snapshots:
                            msg = self._create_stream_beginning_message(
                                test_obj._stream, default_clock_snapshot=self._at
                            )
                        else:
                            msg = self._create_stream_beginning_message(
                                test_obj._stream
                            )
                        test_obj.assertIs(type(msg), bt2._StreamBeginningMessage)
                    elif self._at == 1:
                        msg = self._create_packet_beginning_message(
                            test_obj._packet, self._at
                        )
                        test_obj.assertIs(type(msg), bt2._PacketBeginningMessage)
                    elif self._at == 2:
                        msg = self._create_event_message(
                            test_obj._event_class, test_obj._packet, self._at
                        )
                        test_obj.assertIs(type(msg), bt2._EventMessage)
                    elif self._at == 3:
                        msg = self._create_message_iterator_inactivity_message(
                            test_obj._clock_class, self._at
                        )
                    elif self._at == 4:
                        msg = self._create_discarded_events_message(
                            test_obj._stream, 890, self._at, self._at
                        )
                        test_obj.assertIs(type(msg), bt2._DiscardedEventsMessage)
                    elif self._at == 5:
                        msg = self._create_packet_end_message(
                            test_obj._packet, self._at
                        )
                        test_obj.assertIs(type(msg), bt2._PacketEndMessage)
                    elif self._at == 6:
                        msg = self._create_discarded_packets_message(
                            test_obj._stream, 678, self._at, self._at
                        )
                        test_obj.assertIs(type(msg), bt2._DiscardedPacketsMessage)
                    elif self._at == 7:
                        if self._with_stream_msgs_clock_snapshots:
                            msg = self._create_stream_end_message(
                                test_obj._stream, default_clock_snapshot=self._at
                            )
                        else:
                            msg = self._create_stream_end_message(test_obj._stream)
                        test_obj.assertIs(type(msg), bt2._StreamEndMessage)
                    elif self._at >= 8:
                        raise bt2.Stop
                else:
                    if self._at == 0:
                        msg = self._create_stream_beginning_message(test_obj._stream)
                    elif self._at == 1:
                        msg = self._create_packet_beginning_message(test_obj._packet)
                    elif self._at == 2:
                        msg = self._create_event_message(
                            test_obj._event_class, test_obj._packet
                        )
                    elif self._at == 3:
                        msg = self._create_discarded_events_message(
                            test_obj._stream, 890
                        )
                    elif self._at == 4:
                        msg = self._create_packet_end_message(test_obj._packet)
                    elif self._at == 5:
                        msg = self._create_discarded_packets_message(
                            test_obj._stream, 678
                        )
                    elif self._at == 6:
                        msg = self._create_stream_end_message(test_obj._stream)
                    elif self._at >= 7:
                        raise bt2.Stop

                self._at += 1
                return msg

        class MySrc(bt2._UserSourceComponent, message_iterator_class=MyIter):
            def __init__(self, config, params, obj):
                self._add_output_port('out', params)

                with_cc = bool(params['with_cc'])
                tc = self._create_trace_class()
                if with_cc:
                    cc = self._create_clock_class()
                else:
                    cc = None

                sc = tc.create_stream_class(
                    default_clock_class=cc,
                    supports_packets=True,
                    packets_have_beginning_default_clock_snapshot=with_cc,
                    packets_have_end_default_clock_snapshot=with_cc,
                    supports_discarded_events=True,
                    discarded_events_have_default_clock_snapshots=with_cc,
                    supports_discarded_packets=True,
                    discarded_packets_have_default_clock_snapshots=with_cc,
                )

                # Create payload field class
                my_int_fc = tc.create_signed_integer_field_class(32)
                payload_fc = tc.create_structure_field_class()
                payload_fc += [('my_int', my_int_fc)]

                # Create specific context field class
                my_int_fc = tc.create_signed_integer_field_class(32)
                specific_fc = tc.create_structure_field_class()
                specific_fc += [('my_int', my_int_fc)]

                ec = sc.create_event_class(
                    name='salut',
                    payload_field_class=payload_fc,
                    specific_context_field_class=specific_fc,
                )

                trace = tc()
                stream = trace.create_stream(sc)
                packet = stream.create_packet()

                test_obj._trace = trace
                test_obj._stream = stream
                test_obj._packet = packet
                test_obj._event_class = ec
                test_obj._clock_class = cc

        test_obj = self
        self._graph = bt2.Graph()
        self._src = MySrc
        self._iter = MyIter

    def test_all_msg_with_cc(self):
        params = {'with_cc': True}
        self._src_comp = self._graph.add_component(self._src, 'my_source', params)
        self._msg_iter = TestOutputPortMessageIterator(
            self._graph, self._src_comp.output_ports['out']
        )

        for i, msg in enumerate(self._msg_iter):
            if i == 0:
                self.assertIs(type(msg), bt2._StreamBeginningMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertEqual(msg.stream.addr, self._stream.addr)
                self.assertIsInstance(
                    msg.default_clock_snapshot, bt2._UnknownClockSnapshot
                )
            elif i == 1:
                self.assertIs(type(msg), bt2._PacketBeginningMessageConst)
                self.assertIs(type(msg.packet), bt2_packet._PacketConst)
                self.assertIs(
                    type(msg.default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )
                self.assertEqual(msg.packet.addr, self._packet.addr)
                self.assertEqual(msg.default_clock_snapshot.value, i)
            elif i == 2:
                self.assertIs(type(msg), bt2._EventMessageConst)
                self.assertIs(type(msg.event), bt2_event._EventConst)
                self.assertIs(
                    type(msg.default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )
                self.assertIs(
                    type(msg.event.payload_field), bt2_field._StructureFieldConst
                )
                self.assertIs(
                    type(msg.event.payload_field['my_int']),
                    bt2_field._SignedIntegerFieldConst,
                )

                self.assertEqual(msg.event.cls.addr, self._event_class.addr)
                self.assertEqual(msg.default_clock_snapshot.value, i)
            elif i == 3:
                self.assertIs(type(msg), bt2._MessageIteratorInactivityMessageConst)
                self.assertIs(
                    type(msg.clock_snapshot), bt2_clock_snapshot._ClockSnapshotConst
                )
                self.assertEqual(msg.clock_snapshot.value, i)
            elif i == 4:
                self.assertIs(type(msg), bt2._DiscardedEventsMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertIs(type(msg.stream.cls), bt2_stream_class._StreamClassConst)
                self.assertIs(
                    type(msg.beginning_default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )
                self.assertIs(
                    type(msg.end_default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )

                self.assertEqual(msg.stream.addr, self._stream.addr)
                self.assertEqual(msg.count, 890)
                self.assertEqual(
                    msg.stream.cls.default_clock_class.addr, self._clock_class.addr
                )
                self.assertEqual(msg.beginning_default_clock_snapshot.value, i)
                self.assertEqual(msg.end_default_clock_snapshot.value, i)
            elif i == 5:
                self.assertIs(type(msg), bt2._PacketEndMessageConst)
                self.assertIs(type(msg.packet), bt2_packet._PacketConst)
                self.assertIs(
                    type(msg.default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )
                self.assertEqual(msg.packet.addr, self._packet.addr)
                self.assertEqual(msg.default_clock_snapshot.value, i)
            elif i == 6:
                self.assertIs(type(msg), bt2._DiscardedPacketsMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertIs(type(msg.stream.trace), bt2_trace._TraceConst)
                self.assertIs(
                    type(msg.stream.trace.cls), bt2_trace_class._TraceClassConst
                )
                self.assertIs(
                    type(msg.beginning_default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )
                self.assertIs(
                    type(msg.end_default_clock_snapshot),
                    bt2_clock_snapshot._ClockSnapshotConst,
                )
                self.assertEqual(msg.stream.addr, self._stream.addr)
                self.assertEqual(msg.count, 678)
                self.assertEqual(
                    msg.stream.cls.default_clock_class.addr, self._clock_class.addr
                )
                self.assertEqual(msg.beginning_default_clock_snapshot.value, i)
                self.assertEqual(msg.end_default_clock_snapshot.value, i)
            elif i == 7:
                self.assertIs(type(msg), bt2._StreamEndMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertEqual(msg.stream.addr, self._stream.addr)
                self.assertIs(
                    type(msg.default_clock_snapshot), bt2._UnknownClockSnapshot
                )
            else:
                raise Exception

    def test_all_msg_without_cc(self):
        params = {'with_cc': False}
        self._src_comp = self._graph.add_component(self._src, 'my_source', params)
        self._msg_iter = TestOutputPortMessageIterator(
            self._graph, self._src_comp.output_ports['out']
        )

        for i, msg in enumerate(self._msg_iter):
            if i == 0:
                self.assertIsInstance(msg, bt2._StreamBeginningMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertEqual(msg.stream.addr, self._stream.addr)
                with self.assertRaisesRegex(
                    ValueError, 'stream class has no default clock class'
                ):
                    msg.default_clock_snapshot
            elif i == 1:
                self.assertIsInstance(msg, bt2._PacketBeginningMessageConst)
                self.assertIs(type(msg.packet), bt2_packet._PacketConst)
                self.assertEqual(msg.packet.addr, self._packet.addr)
            elif i == 2:
                self.assertIsInstance(msg, bt2._EventMessageConst)
                self.assertIs(type(msg.event), bt2_event._EventConst)
                self.assertIs(type(msg.event.cls), bt2_event_class._EventClassConst)
                self.assertEqual(msg.event.cls.addr, self._event_class.addr)
                with self.assertRaisesRegex(
                    ValueError, 'stream class has no default clock class'
                ):
                    msg.default_clock_snapshot
            elif i == 3:
                self.assertIsInstance(msg, bt2._DiscardedEventsMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertIs(type(msg.stream.cls), bt2_stream_class._StreamClassConst)
                self.assertEqual(msg.stream.addr, self._stream.addr)
                self.assertEqual(msg.count, 890)
                self.assertIsNone(msg.stream.cls.default_clock_class)
                with self.assertRaisesRegex(
                    ValueError,
                    'such a message has no clock snapshots for this stream class',
                ):
                    msg.beginning_default_clock_snapshot
                with self.assertRaisesRegex(
                    ValueError,
                    'such a message has no clock snapshots for this stream class',
                ):
                    msg.end_default_clock_snapshot
            elif i == 4:
                self.assertIsInstance(msg, bt2._PacketEndMessageConst)
                self.assertEqual(msg.packet.addr, self._packet.addr)
                self.assertIs(type(msg.packet), bt2_packet._PacketConst)
            elif i == 5:
                self.assertIsInstance(msg, bt2._DiscardedPacketsMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertIs(type(msg.stream.cls), bt2_stream_class._StreamClassConst)
                self.assertIs(
                    type(msg.stream.cls.trace_class), bt2_trace_class._TraceClassConst
                )
                self.assertEqual(msg.stream.addr, self._stream.addr)
                self.assertEqual(msg.count, 678)
                self.assertIsNone(msg.stream.cls.default_clock_class)
                with self.assertRaisesRegex(
                    ValueError,
                    'such a message has no clock snapshots for this stream class',
                ):
                    msg.beginning_default_clock_snapshot
                with self.assertRaisesRegex(
                    ValueError,
                    'such a message has no clock snapshots for this stream class',
                ):
                    msg.end_default_clock_snapshot
            elif i == 6:
                self.assertIsInstance(msg, bt2._StreamEndMessageConst)
                self.assertIs(type(msg.stream), bt2_stream._StreamConst)
                self.assertEqual(msg.stream.addr, self._stream.addr)
                with self.assertRaisesRegex(
                    ValueError, 'stream class has no default clock class'
                ):
                    msg.default_clock_snapshot
            else:
                raise Exception

    def test_msg_stream_with_clock_snapshots(self):
        params = {'with_cc': True, 'with_stream_msgs_clock_snapshots': True}

        self._src_comp = self._graph.add_component(self._src, 'my_source', params)
        self._msg_iter = TestOutputPortMessageIterator(
            self._graph, self._src_comp.output_ports['out']
        )
        msgs = list(self._msg_iter)

        msg_stream_beg = msgs[0]
        self.assertIsInstance(msg_stream_beg, bt2._StreamBeginningMessageConst)
        self.assertIs(
            type(msg_stream_beg.default_clock_snapshot),
            bt2_clock_snapshot._ClockSnapshotConst,
        )
        self.assertEqual(msg_stream_beg.default_clock_snapshot.value, 0)

        msg_stream_end = msgs[7]
        self.assertIsInstance(msg_stream_end, bt2._StreamEndMessageConst)
        self.assertIs(
            type(msg_stream_end.default_clock_snapshot),
            bt2_clock_snapshot._ClockSnapshotConst,
        )
        self.assertEqual(msg_stream_end.default_clock_snapshot.value, 7)

    def test_stream_beg_msg(self):
        msg = utils.get_stream_beginning_message()
        self.assertIs(type(msg.stream), bt2_stream._Stream)

    def test_stream_end_msg(self):
        msg = utils.get_stream_end_message()
        self.assertIs(type(msg.stream), bt2_stream._Stream)

    def test_packet_beg_msg(self):
        msg = utils.get_packet_beginning_message()
        self.assertIs(type(msg.packet), bt2_packet._Packet)

    def test_packet_end_msg(self):
        msg = utils.get_packet_end_message()
        self.assertIs(type(msg.packet), bt2_packet._Packet)

    def test_event_msg(self):
        msg = utils.get_event_message()
        self.assertIs(type(msg.event), bt2_event._Event)


class CreateDiscardedEventMessageTestCase(unittest.TestCase):
    # Most basic case.
    def test_create(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(supports_discarded_events=True)

        def msg_iter_next(msg_iter, stream):
            return msg_iter._create_discarded_events_message(stream)

        msg = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertIs(type(msg), bt2._DiscardedEventsMessage)
        self.assertIs(msg.count, None)

    # With event count.
    def test_create_with_count(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(supports_discarded_events=True)

        def msg_iter_next(msg_iter, stream):
            return msg_iter._create_discarded_events_message(stream, count=242)

        msg = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertIs(type(msg), bt2._DiscardedEventsMessage)
        self.assertEqual(msg.count, 242)

    # With clock snapshots.
    def test_create_with_clock_snapshots(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                default_clock_class=cc,
                supports_discarded_events=True,
                discarded_events_have_default_clock_snapshots=True,
            )

        def msg_iter_next(msg_iter, stream):
            return msg_iter._create_discarded_events_message(
                stream, beg_clock_snapshot=10, end_clock_snapshot=20
            )

        msg = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertIs(type(msg), bt2._DiscardedEventsMessage)
        self.assertEqual(msg.beginning_default_clock_snapshot, 10)
        self.assertEqual(msg.end_default_clock_snapshot, 20)

    # Trying to create when the stream does not support discarded events.
    def test_create_unsupported_raises(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class()

        def msg_iter_next(msg_iter, stream):
            with self.assertRaisesRegex(
                ValueError, 'stream class does not support discarded events'
            ):
                msg_iter._create_discarded_events_message(stream)

            return 123

        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertEqual(res, 123)

    # Trying to create with clock snapshots when the stream does not support
    # them.
    def test_create_unsupported_clock_snapshots_raises(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(supports_discarded_events=True)

        def msg_iter_next(msg_iter, stream):
            with self.assertRaisesRegex(
                ValueError,
                'discarded events have no default clock snapshots for this stream class',
            ):
                msg_iter._create_discarded_events_message(
                    stream, beg_clock_snapshot=10, end_clock_snapshot=20
                )

            return 123

        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertEqual(res, 123)

    # Trying to create without clock snapshots when the stream requires them.
    def test_create_missing_clock_snapshots_raises(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                default_clock_class=cc,
                supports_discarded_events=True,
                discarded_events_have_default_clock_snapshots=True,
            )

        def msg_iter_next(msg_iter, stream):
            with self.assertRaisesRegex(
                ValueError,
                'discarded events have default clock snapshots for this stream class',
            ):
                msg_iter._create_discarded_events_message(stream)

            return 123

        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertEqual(res, 123)


class CreateDiscardedPacketMessageTestCase(unittest.TestCase):
    # Most basic case.
    def test_create(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                supports_packets=True, supports_discarded_packets=True
            )

        def msg_iter_next(msg_iter, stream):
            return msg_iter._create_discarded_packets_message(stream)

        msg = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertIs(type(msg), bt2._DiscardedPacketsMessage)
        self.assertIs(msg.count, None)

    # With packet count.
    def test_create_with_count(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                supports_packets=True, supports_discarded_packets=True
            )

        def msg_iter_next(msg_iter, stream):
            return msg_iter._create_discarded_packets_message(stream, count=242)

        msg = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertIs(type(msg), bt2._DiscardedPacketsMessage)
        self.assertEqual(msg.count, 242)

    # With clock snapshots.
    def test_create_with_clock_snapshots(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                default_clock_class=cc,
                supports_packets=True,
                supports_discarded_packets=True,
                discarded_packets_have_default_clock_snapshots=True,
            )

        def msg_iter_next(msg_iter, stream):
            return msg_iter._create_discarded_packets_message(
                stream, beg_clock_snapshot=10, end_clock_snapshot=20
            )

        msg = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertIs(type(msg), bt2._DiscardedPacketsMessage)
        self.assertEqual(msg.beginning_default_clock_snapshot, 10)
        self.assertEqual(msg.end_default_clock_snapshot, 20)

    # Trying to create when the stream does not support discarded packets.
    def test_create_unsupported_raises(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                supports_packets=True,
            )

        def msg_iter_next(msg_iter, stream):
            with self.assertRaisesRegex(
                ValueError, 'stream class does not support discarded packets'
            ):
                msg_iter._create_discarded_packets_message(stream)

            return 123

        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertEqual(res, 123)

    # Trying to create with clock snapshots when the stream does not support
    # them.
    def test_create_unsupported_clock_snapshots_raises(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                supports_packets=True, supports_discarded_packets=True
            )

        def msg_iter_next(msg_iter, stream):
            with self.assertRaisesRegex(
                ValueError,
                'discarded packets have no default clock snapshots for this stream class',
            ):
                msg_iter._create_discarded_packets_message(
                    stream, beg_clock_snapshot=10, end_clock_snapshot=20
                )

            return 123

        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertEqual(res, 123)

    # Trying to create without clock snapshots when the stream requires them.
    def test_create_missing_clock_snapshots_raises(self):
        def create_stream_class(tc, cc):
            return tc.create_stream_class(
                default_clock_class=cc,
                supports_packets=True,
                supports_discarded_packets=True,
                discarded_packets_have_default_clock_snapshots=True,
            )

        def msg_iter_next(msg_iter, stream):
            with self.assertRaisesRegex(
                ValueError,
                'discarded packets have default clock snapshots for this stream class',
            ):
                msg_iter._create_discarded_packets_message(stream)

            return 123

        res = utils.run_in_message_iterator_next(create_stream_class, msg_iter_next)
        self.assertEqual(res, 123)


if __name__ == '__main__':
    unittest.main()



import bt2


class TheIteratorOfAllEvil(bt2._UserMessageIterator):
    def __init__(self, config, port):
        tc, sc, ec1, ec2, params = port.user_data
        trace = tc()
        stream = trace.create_stream(sc)

        if params['with-packet-msgs']:
            packet = stream.create_packet()

        if params['with-stream-msgs-cs']:
            sb_msg = self._create_stream_beginning_message(stream, 100)
        else:
            sb_msg = self._create_stream_beginning_message(stream)

        parent = packet if params['with-packet-msgs'] else stream

        ev_msg1 = self._create_event_message(ec1, parent, 300)
        ev_msg2 = self._create_event_message(ec2, parent, 400)

        if params['with-stream-msgs-cs']:
            se_msg = self._create_stream_end_message(stream, 1000)
        else:
            se_msg = self._create_stream_end_message(stream)

        self._msgs = []

        self._msgs.append(sb_msg)

        if params['with-packet-msgs']:
            self._msgs.append(self._create_packet_beginning_message(packet, 200))

        self._msgs.append(ev_msg1)
        self._msgs.append(ev_msg2)

        if params['with-packet-msgs']:
            self._msgs.append(self._create_packet_end_message(packet, 900))

        self._msgs.append(se_msg)

        self._at = 0
        config.can_seek_forward = True

    def _user_seek_beginning(self):
        self._at = 0

    def __next__(self):
        if self._at < len(self._msgs):
            msg = self._msgs[self._at]
            self._at += 1
            return msg
        else:
            raise StopIteration


@bt2.plugin_component_class
class TheSourceOfAllEvil(
    bt2._UserSourceComponent, message_iterator_class=TheIteratorOfAllEvil
):
    def __init__(self, config, params, obj):
        tc = self._create_trace_class()

        with_packets = bool(params['with-packet-msgs'])

        # Use a clock class with an offset, so we can test with --begin or --end
        # smaller than this offset (in other words, a time that it's not
        # possible to represent with this clock class).
        cc = self._create_clock_class(frequency=1, offset=bt2.ClockClassOffset(10000))
        sc = tc.create_stream_class(
            default_clock_class=cc,
            supports_packets=with_packets,
            packets_have_beginning_default_clock_snapshot=with_packets,
            packets_have_end_default_clock_snapshot=with_packets,
        )
        ec1 = sc.create_event_class(name='event 1')
        ec2 = sc.create_event_class(name='event 2')
        self._add_output_port('out', (tc, sc, ec1, ec2, params))


bt2.register_plugin(__name__, 'test-trimmer')



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

from functools import partial, partialmethod
import operator
import collections
import unittest
import math
import copy
import bt2


# The value object classes explicitly do not implement the copy methods,
# raising `NotImplementedError`, just in case we decide to implement
# them someday.
class _TestCopySimple:
    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            copy.copy(self._def)

    def test_deepcopy(self):
        with self.assertRaises(NotImplementedError):
            copy.deepcopy(self._def)


_COMP_BINOPS = (operator.eq, operator.ne)


# Base class for numeric value test cases.
#
# To be compatible with this base class, a derived class must, in its
# setUp() method:
#
# * Set `self._def` to a value object with an arbitrary raw value.
# * Set `self._def_value` to the equivalent raw value of `self._def`.
class _TestNumericValue(_TestCopySimple):
    # Tries the binary operation `op`:
    #
    # 1. Between `self._def`, which is a value object, and `rhs`.
    # 2. Between `self._def_value`, which is the raw value of
    #    `self._def`, and `rhs`.
    #
    # Returns the results of 1. and 2.
    #
    # If there's an exception while performing 1. or 2., asserts that
    # both operations raised exceptions, that both exceptions have the
    # same type, and returns `None` for both results.
    def _binop(self, op, rhs):
        type_rexc = None
        type_rvexc = None
        comp_value = rhs

        # try with value object
        try:
            r = op(self._def, rhs)
        except Exception as e:
            type_rexc = type(e)

        # try with raw value
        try:
            rv = op(self._def_value, comp_value)
        except Exception as e:
            type_rvexc = type(e)

        if type_rexc is not None or type_rvexc is not None:
            # at least one of the operations raised an exception: in
            # this case both operations should have raised the same
            # type of exception (division by zero, bit shift with a
            # floating point number operand, etc.)
            self.assertIs(type_rexc, type_rvexc)
            return None, None

        return r, rv

    # Tries the unary operation `op`:
    #
    # 1. On `self._def`, which is a value object.
    # 2. On `self._def_value`, which is the raw value of `self._def`.
    #
    # Returns the results of 1. and 2.
    #
    # If there's an exception while performing 1. or 2., asserts that
    # both operations raised exceptions, that both exceptions have the
    # same type, and returns `None` for both results.
    def _unaryop(self, op):
        type_rexc = None
        type_rvexc = None

        # try with value object
        try:
            r = op(self._def)
        except Exception as e:
            type_rexc = type(e)

        # try with raw value
        try:
            rv = op(self._def_value)
        except Exception as e:
            type_rvexc = type(e)

        if type_rexc is not None or type_rvexc is not None:
            # at least one of the operations raised an exception: in
            # this case both operations should have raised the same
            # type of exception (division by zero, bit shift with a
            # floating point number operand, etc.)
            self.assertIs(type_rexc, type_rvexc)
            return None, None

        return r, rv

    # Tests that the unary operation `op` gives results with the same
    # type for both `self._def` and `self._def_value`.
    def _test_unaryop_type(self, op):
        r, rv = self._unaryop(op)

        if r is None:
            return

        self.assertIsInstance(r, type(rv))

    # Tests that the unary operation `op` gives results with the same
    # value for both `self._def` and `self._def_value`. This uses the
    # __eq__() operator of `self._def`.
    def _test_unaryop_value(self, op):
        r, rv = self._unaryop(op)

        if r is None:
            return

        self.assertEqual(r, rv)

    # Tests that the unary operation `op`, when applied to `self._def`,
    # does not change its underlying BT object address.
    def _test_unaryop_addr_same(self, op):
        addr_before = self._def.addr
        self._unaryop(op)
        self.assertEqual(self._def.addr, addr_before)

    # Tests that the unary operation `op`, when applied to `self._def`,
    # does not change its value.
    def _test_unaryop_value_same(self, op):
        value_before = self._def.__class__(self._def)
        self._unaryop(op)
        self.assertEqual(self._def, value_before)

    # Tests that the binary operation `op` gives results with the same
    # type for both `self._def` and `self._def_value`.
    def _test_binop_type(self, op, rhs):
        r, rv = self._binop(op, rhs)

        if r is None:
            return

        if op in _COMP_BINOPS:
            # __eq__() and __ne__() always return a 'bool' object
            self.assertIsInstance(r, bool)
        else:
            self.assertIsInstance(r, type(rv))

    # Tests that the binary operation `op` gives results with the same
    # value for both `self._def` and `self._def_value`. This uses the
    # __eq__() operator of `self._def`.
    def _test_binop_value(self, op, rhs):
        r, rv = self._binop(op, rhs)

        if r is None:
            return

        self.assertEqual(r, rv)

    # Tests that the binary operation `op`, when applied to `self._def`,
    # does not change its underlying BT object address.
    def _test_binop_lhs_addr_same(self, op, rhs):
        addr_before = self._def.addr
        r, rv = self._binop(op, rhs)
        self.assertEqual(self._def.addr, addr_before)

    # Tests that the binary operation `op`, when applied to `self._def`,
    # does not change its value.
    def _test_binop_lhs_value_same(self, op, rhs):
        value_before = self._def.__class__(self._def)
        r, rv = self._binop(op, rhs)
        self.assertEqual(self._def, value_before)

    # The methods below which take the `test_cb` and `op` parameters
    # are meant to be used with one of the _test_binop_*() functions
    # above as `test_cb` and a binary operator function as `op`.
    #
    # For example:
    #
    #     self._test_binop_rhs_pos_int(self._test_binop_value,
    #                                  operator.add)
    #
    # This tests that a numeric value object added to a positive integer
    # raw value gives a result with the expected value.
    #
    # `vint` and `vfloat` mean a signed integer value object and a real
    # value object.

    def _test_binop_unknown(self, op):
        if op is operator.eq:
            self.assertIs(op(self._def, object()), False)
        elif op is operator.ne:
            self.assertIs(op(self._def, object()), True)
        else:
            with self.assertRaises(TypeError):
                op(self._def, object())

    def _test_binop_none(self, op):
        if op is operator.eq:
            self.assertIs(op(self._def, None), False)
        elif op is operator.ne:
            self.assertIs(op(self._def, None), True)
        else:
            with self.assertRaises(TypeError):
                op(self._def, None)

    def _test_binop_rhs_false(self, test_cb, op):
        test_cb(op, False)

    def _test_binop_rhs_true(self, test_cb, op):
        test_cb(op, True)

    def _test_binop_rhs_pos_int(self, test_cb, op):
        test_cb(op, 2)

    def _test_binop_rhs_neg_int(self, test_cb, op):
        test_cb(op, -23)

    def _test_binop_rhs_zero_int(self, test_cb, op):
        test_cb(op, 0)

    def _test_binop_rhs_pos_vint(self, test_cb, op):
        test_cb(op, bt2.create_value(2))

    def _test_binop_rhs_neg_vint(self, test_cb, op):
        test_cb(op, bt2.create_value(-23))

    def _test_binop_rhs_zero_vint(self, test_cb, op):
        test_cb(op, bt2.create_value(0))

    def _test_binop_rhs_pos_float(self, test_cb, op):
        test_cb(op, 2.2)

    def _test_binop_rhs_neg_float(self, test_cb, op):
        test_cb(op, -23.4)

    def _test_binop_rhs_zero_float(self, test_cb, op):
        test_cb(op, 0.0)

    def _test_binop_rhs_complex(self, test_cb, op):
        test_cb(op, -23 + 19j)

    def _test_binop_rhs_zero_complex(self, test_cb, op):
        test_cb(op, 0j)

    def _test_binop_rhs_pos_vfloat(self, test_cb, op):
        test_cb(op, bt2.create_value(2.2))

    def _test_binop_rhs_neg_vfloat(self, test_cb, op):
        test_cb(op, bt2.create_value(-23.4))

    def _test_binop_rhs_zero_vfloat(self, test_cb, op):
        test_cb(op, bt2.create_value(0.0))

    def _test_binop_type_false(self, op):
        self._test_binop_rhs_false(self._test_binop_type, op)

    def _test_binop_type_true(self, op):
        self._test_binop_rhs_true(self._test_binop_type, op)

    def _test_binop_type_pos_int(self, op):
        self._test_binop_rhs_pos_int(self._test_binop_type, op)

    def _test_binop_type_neg_int(self, op):
        self._test_binop_rhs_neg_int(self._test_binop_type, op)

    def _test_binop_type_zero_int(self, op):
        self._test_binop_rhs_zero_int(self._test_binop_type, op)

    def _test_binop_type_pos_vint(self, op):
        self._test_binop_rhs_pos_vint(self._test_binop_type, op)

    def _test_binop_type_neg_vint(self, op):
        self._test_binop_rhs_neg_vint(self._test_binop_type, op)

    def _test_binop_type_zero_vint(self, op):
        self._test_binop_rhs_zero_vint(self._test_binop_type, op)

    def _test_binop_type_pos_float(self, op):
        self._test_binop_rhs_pos_float(self._test_binop_type, op)

    def _test_binop_type_neg_float(self, op):
        self._test_binop_rhs_neg_float(self._test_binop_type, op)

    def _test_binop_type_zero_float(self, op):
        self._test_binop_rhs_zero_float(self._test_binop_type, op)

    def _test_binop_type_pos_vfloat(self, op):
        self._test_binop_rhs_pos_vfloat(self._test_binop_type, op)

    def _test_binop_type_neg_vfloat(self, op):
        self._test_binop_rhs_neg_vfloat(self._test_binop_type, op)

    def _test_binop_type_zero_vfloat(self, op):
        self._test_binop_rhs_zero_vfloat(self._test_binop_type, op)

    def _test_binop_type_complex(self, op):
        self._test_binop_rhs_complex(self._test_binop_type, op)

    def _test_binop_type_zero_complex(self, op):
        self._test_binop_rhs_zero_complex(self._test_binop_type, op)

    def _test_binop_value_false(self, op):
        self._test_binop_rhs_false(self._test_binop_value, op)

    def _test_binop_value_true(self, op):
        self._test_binop_rhs_true(self._test_binop_value, op)

    def _test_binop_value_pos_int(self, op):
        self._test_binop_rhs_pos_int(self._test_binop_value, op)

    def _test_binop_value_neg_int(self, op):
        self._test_binop_rhs_neg_int(self._test_binop_value, op)

    def _test_binop_value_zero_int(self, op):
        self._test_binop_rhs_zero_int(self._test_binop_value, op)

    def _test_binop_value_pos_vint(self, op):
        self._test_binop_rhs_pos_vint(self._test_binop_value, op)

    def _test_binop_value_neg_vint(self, op):
        self._test_binop_rhs_neg_vint(self._test_binop_value, op)

    def _test_binop_value_zero_vint(self, op):
        self._test_binop_rhs_zero_vint(self._test_binop_value, op)

    def _test_binop_value_pos_float(self, op):
        self._test_binop_rhs_pos_float(self._test_binop_value, op)

    def _test_binop_value_neg_float(self, op):
        self._test_binop_rhs_neg_float(self._test_binop_value, op)

    def _test_binop_value_zero_float(self, op):
        self._test_binop_rhs_zero_float(self._test_binop_value, op)

    def _test_binop_value_pos_vfloat(self, op):
        self._test_binop_rhs_pos_vfloat(self._test_binop_value, op)

    def _test_binop_value_neg_vfloat(self, op):
        self._test_binop_rhs_neg_vfloat(self._test_binop_value, op)

    def _test_binop_value_zero_vfloat(self, op):
        self._test_binop_rhs_zero_vfloat(self._test_binop_value, op)

    def _test_binop_value_complex(self, op):
        self._test_binop_rhs_complex(self._test_binop_value, op)

    def _test_binop_value_zero_complex(self, op):
        self._test_binop_rhs_zero_complex(self._test_binop_value, op)

    def _test_binop_lhs_addr_same_false(self, op):
        self._test_binop_rhs_false(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_true(self, op):
        self._test_binop_rhs_true(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_pos_int(self, op):
        self._test_binop_rhs_pos_int(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_neg_int(self, op):
        self._test_binop_rhs_neg_int(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_zero_int(self, op):
        self._test_binop_rhs_zero_int(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_pos_vint(self, op):
        self._test_binop_rhs_pos_vint(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_neg_vint(self, op):
        self._test_binop_rhs_neg_vint(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_zero_vint(self, op):
        self._test_binop_rhs_zero_vint(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_pos_float(self, op):
        self._test_binop_rhs_pos_float(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_neg_float(self, op):
        self._test_binop_rhs_neg_float(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_zero_float(self, op):
        self._test_binop_rhs_zero_float(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_pos_vfloat(self, op):
        self._test_binop_rhs_pos_vfloat(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_neg_vfloat(self, op):
        self._test_binop_rhs_neg_vfloat(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_zero_vfloat(self, op):
        self._test_binop_rhs_zero_vfloat(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_complex(self, op):
        self._test_binop_rhs_complex(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_addr_same_zero_complex(self, op):
        self._test_binop_rhs_zero_complex(self._test_binop_lhs_addr_same, op)

    def _test_binop_lhs_value_same_false(self, op):
        self._test_binop_rhs_false(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_true(self, op):
        self._test_binop_rhs_true(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_pos_int(self, op):
        self._test_binop_rhs_pos_int(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_neg_int(self, op):
        self._test_binop_rhs_neg_int(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_zero_int(self, op):
        self._test_binop_rhs_zero_int(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_pos_vint(self, op):
        self._test_binop_rhs_pos_vint(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_neg_vint(self, op):
        self._test_binop_rhs_neg_vint(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_zero_vint(self, op):
        self._test_binop_rhs_zero_vint(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_pos_float(self, op):
        self._test_binop_rhs_pos_float(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_neg_float(self, op):
        self._test_binop_rhs_neg_float(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_zero_float(self, op):
        self._test_binop_rhs_zero_float(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_pos_vfloat(self, op):
        self._test_binop_rhs_pos_vfloat(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_neg_vfloat(self, op):
        self._test_binop_rhs_neg_vfloat(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_zero_vfloat(self, op):
        self._test_binop_rhs_zero_vfloat(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_complex(self, op):
        self._test_binop_rhs_complex(self._test_binop_lhs_value_same, op)

    def _test_binop_lhs_value_same_zero_complex(self, op):
        self._test_binop_rhs_zero_complex(self._test_binop_lhs_value_same, op)

    def test_bool_op(self):
        self.assertEqual(bool(self._def), bool(self._def_value))

    def test_int_op(self):
        self.assertEqual(int(self._def), int(self._def_value))

    def test_float_op(self):
        self.assertEqual(float(self._def), float(self._def_value))

    def test_complex_op(self):
        self.assertEqual(complex(self._def), complex(self._def_value))

    def test_str_op(self):
        self.assertEqual(str(self._def), str(self._def_value))

    def test_eq_none(self):
        # Disable the "comparison to None" warning, as this is precisely what
        # we want to test here.
        self.assertFalse(self._def == None)  # noqa: E711

    def test_ne_none(self):
        # Disable the "comparison to None" warning, as this is precisely what
        # we want to test here.
        self.assertTrue(self._def != None)  # noqa: E711


# This is a list of binary operators used for
# _inject_numeric_testing_methods().
#
# Each entry is a pair of binary operator name (used as part of the
# created testing method's name) and operator function.
_BINOPS = (
    ('lt', operator.lt),
    ('le', operator.le),
    ('eq', operator.eq),
    ('ne', operator.ne),
    ('ge', operator.ge),
    ('gt', operator.gt),
    ('add', operator.add),
    ('radd', lambda a, b: operator.add(b, a)),
    ('and', operator.and_),
    ('rand', lambda a, b: operator.and_(b, a)),
    ('floordiv', operator.floordiv),
    ('rfloordiv', lambda a, b: operator.floordiv(b, a)),
    ('lshift', operator.lshift),
    ('rlshift', lambda a, b: operator.lshift(b, a)),
    ('mod', operator.mod),
    ('rmod', lambda a, b: operator.mod(b, a)),
    ('mul', operator.mul),
    ('rmul', lambda a, b: operator.mul(b, a)),
    ('or', operator.or_),
    ('ror', lambda a, b: operator.or_(b, a)),
    ('pow', operator.pow),
    ('rpow', lambda a, b: operator.pow(b, a)),
    ('rshift', operator.rshift),
    ('rrshift', lambda a, b: operator.rshift(b, a)),
    ('sub', operator.sub),
    ('rsub', lambda a, b: operator.sub(b, a)),
    ('truediv', operator.truediv),
    ('rtruediv', lambda a, b: operator.truediv(b, a)),
    ('xor', operator.xor),
    ('rxor', lambda a, b: operator.xor(b, a)),
)


# This is a list of unary operators used for
# _inject_numeric_testing_methods().
#
# Each entry is a pair of unary operator name (used as part of the
# created testing method's name) and operator function.
_UNARYOPS = (
    ('neg', operator.neg),
    ('pos', operator.pos),
    ('abs', operator.abs),
    ('invert', operator.invert),
    ('round', round),
    ('round_0', partial(round, ndigits=0)),
    ('round_1', partial(round, ndigits=1)),
    ('round_2', partial(round, ndigits=2)),
    ('round_3', partial(round, ndigits=3)),
    ('ceil', math.ceil),
    ('floor', math.floor),
    ('trunc', math.trunc),
)


# This function injects a bunch of testing methods to a numeric
# value test case.
#
# It is meant to be used like this:
#
#     _inject_numeric_testing_methods(MyNumericValueTestCase)
#
# This function injects:
#
# * One testing method for each _TestNumericValue._test_binop_*()
#   method, for each binary operator in the _BINOPS tuple.
#
# * One testing method for each _TestNumericValue._test_unaryop*()
#   method, for each unary operator in the _UNARYOPS tuple.
def _inject_numeric_testing_methods(cls):
    def test_binop_name(suffix):
        return 'test_binop_{}_{}'.format(name, suffix)

    def test_unaryop_name(suffix):
        return 'test_unaryop_{}_{}'.format(name, suffix)

    # inject testing methods for each binary operation
    for name, binop in _BINOPS:
        setattr(
            cls,
            test_binop_name('unknown'),
            partialmethod(_TestNumericValue._test_binop_unknown, op=binop),
        )
        setattr(
            cls,
            test_binop_name('none'),
            partialmethod(_TestNumericValue._test_binop_none, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_true'),
            partialmethod(_TestNumericValue._test_binop_type_true, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_pos_int'),
            partialmethod(_TestNumericValue._test_binop_type_pos_int, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_pos_vint'),
            partialmethod(_TestNumericValue._test_binop_type_pos_vint, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_true'),
            partialmethod(_TestNumericValue._test_binop_value_true, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_pos_int'),
            partialmethod(_TestNumericValue._test_binop_value_pos_int, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_pos_vint'),
            partialmethod(_TestNumericValue._test_binop_value_pos_vint, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_true'),
            partialmethod(_TestNumericValue._test_binop_lhs_addr_same_true, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_pos_int'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_pos_int, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_pos_vint'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_pos_vint, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_true'),
            partialmethod(_TestNumericValue._test_binop_lhs_value_same_true, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_pos_int'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_pos_int, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_pos_vint'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_pos_vint, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('type_neg_int'),
            partialmethod(_TestNumericValue._test_binop_type_neg_int, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_neg_vint'),
            partialmethod(_TestNumericValue._test_binop_type_neg_vint, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_neg_int'),
            partialmethod(_TestNumericValue._test_binop_value_neg_int, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_neg_vint'),
            partialmethod(_TestNumericValue._test_binop_value_neg_vint, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_neg_int'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_neg_int, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_neg_vint'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_neg_vint, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_neg_int'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_neg_int, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_neg_vint'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_neg_vint, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('type_false'),
            partialmethod(_TestNumericValue._test_binop_type_false, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_zero_int'),
            partialmethod(_TestNumericValue._test_binop_type_zero_int, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_zero_vint'),
            partialmethod(_TestNumericValue._test_binop_type_zero_vint, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_false'),
            partialmethod(_TestNumericValue._test_binop_value_false, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_zero_int'),
            partialmethod(_TestNumericValue._test_binop_value_zero_int, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_zero_vint'),
            partialmethod(_TestNumericValue._test_binop_value_zero_vint, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_false'),
            partialmethod(_TestNumericValue._test_binop_lhs_addr_same_false, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_zero_int'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_zero_int, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_zero_vint'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_zero_vint, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_false'),
            partialmethod(_TestNumericValue._test_binop_lhs_value_same_false, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_zero_int'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_zero_int, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_zero_vint'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_zero_vint, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('type_neg_float'),
            partialmethod(_TestNumericValue._test_binop_type_neg_float, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_neg_vfloat'),
            partialmethod(_TestNumericValue._test_binop_type_neg_vfloat, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_neg_float'),
            partialmethod(_TestNumericValue._test_binop_value_neg_float, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_neg_vfloat'),
            partialmethod(_TestNumericValue._test_binop_value_neg_vfloat, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_neg_float'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_neg_float, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_neg_vfloat'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_neg_vfloat, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_neg_float'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_neg_float, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_neg_vfloat'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_neg_vfloat, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('type_pos_float'),
            partialmethod(_TestNumericValue._test_binop_type_pos_float, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_pos_vfloat'),
            partialmethod(_TestNumericValue._test_binop_type_pos_vfloat, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_pos_float'),
            partialmethod(_TestNumericValue._test_binop_value_pos_float, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_pos_vfloat'),
            partialmethod(_TestNumericValue._test_binop_value_pos_vfloat, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_pos_float'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_pos_float, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_pos_vfloat'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_pos_vfloat, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_pos_float'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_pos_float, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_pos_vfloat'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_pos_vfloat, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('type_zero_float'),
            partialmethod(_TestNumericValue._test_binop_type_zero_float, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_zero_vfloat'),
            partialmethod(_TestNumericValue._test_binop_type_zero_vfloat, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_zero_float'),
            partialmethod(_TestNumericValue._test_binop_value_zero_float, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_zero_vfloat'),
            partialmethod(_TestNumericValue._test_binop_value_zero_vfloat, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_zero_float'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_zero_float, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_zero_vfloat'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_zero_vfloat, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_zero_float'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_zero_float, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_zero_vfloat'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_zero_vfloat, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('type_complex'),
            partialmethod(_TestNumericValue._test_binop_type_complex, op=binop),
        )
        setattr(
            cls,
            test_binop_name('type_zero_complex'),
            partialmethod(_TestNumericValue._test_binop_type_zero_complex, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_complex'),
            partialmethod(_TestNumericValue._test_binop_value_complex, op=binop),
        )
        setattr(
            cls,
            test_binop_name('value_zero_complex'),
            partialmethod(_TestNumericValue._test_binop_value_zero_complex, op=binop),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_complex'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_complex, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_addr_same_zero_complex'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_addr_same_zero_complex, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_complex'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_complex, op=binop
            ),
        )
        setattr(
            cls,
            test_binop_name('lhs_value_same_zero_complex'),
            partialmethod(
                _TestNumericValue._test_binop_lhs_value_same_zero_complex, op=binop
            ),
        )

    # inject testing methods for each unary operation
    for name, unaryop in _UNARYOPS:
        setattr(
            cls,
            test_unaryop_name('type'),
            partialmethod(_TestNumericValue._test_unaryop_type, op=unaryop),
        )
        setattr(
            cls,
            test_unaryop_name('value'),
            partialmethod(_TestNumericValue._test_unaryop_value, op=unaryop),
        )
        setattr(
            cls,
            test_unaryop_name('addr_same'),
            partialmethod(_TestNumericValue._test_unaryop_addr_same, op=unaryop),
        )
        setattr(
            cls,
            test_unaryop_name('value_same'),
            partialmethod(_TestNumericValue._test_unaryop_value_same, op=unaryop),
        )


class CreateValueFuncTestCase(unittest.TestCase):
    def test_create_none(self):
        v = bt2.create_value(None)
        self.assertIsNone(v)

    def test_create_bool_false(self):
        v = bt2.create_value(False)
        self.assertIsInstance(v, bt2.BoolValue)
        self.assertFalse(v)

    def test_create_bool_true(self):
        v = bt2.create_value(True)
        self.assertIsInstance(v, bt2.BoolValue)
        self.assertTrue(v)

    def test_create_int_pos(self):
        raw = 23
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.SignedIntegerValue)
        self.assertEqual(v, raw)

    def test_create_int_neg(self):
        raw = -23
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.SignedIntegerValue)
        self.assertEqual(v, raw)

    def test_create_float_pos(self):
        raw = 17.5
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.RealValue)
        self.assertEqual(v, raw)

    def test_create_float_neg(self):
        raw = -17.5
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.RealValue)
        self.assertEqual(v, raw)

    def test_create_string(self):
        raw = 'salut'
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.StringValue)
        self.assertEqual(v, raw)

    def test_create_string_empty(self):
        raw = ''
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.StringValue)
        self.assertEqual(v, raw)

    def test_create_array_from_list(self):
        raw = [1, 2, 3]
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.ArrayValue)
        self.assertEqual(v, raw)

    def test_create_array_from_tuple(self):
        raw = 4, 5, 6
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.ArrayValue)
        self.assertEqual(v, raw)

    def test_create_array_from_empty_list(self):
        raw = []
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.ArrayValue)
        self.assertEqual(v, raw)

    def test_create_array_from_empty_tuple(self):
        raw = ()
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.ArrayValue)
        self.assertEqual(v, raw)

    def test_create_map(self):
        raw = {'salut': 23}
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.MapValue)
        self.assertEqual(v, raw)

    def test_create_map_empty(self):
        raw = {}
        v = bt2.create_value(raw)
        self.assertIsInstance(v, bt2.MapValue)
        self.assertEqual(v, raw)

    def test_create_vfalse(self):
        v = bt2.create_value(bt2.create_value(False))
        self.assertIsInstance(v, bt2.BoolValue)
        self.assertFalse(v)

    def test_create_invalid(self):
        class A:
            pass

        a = A()

        with self.assertRaisesRegex(
            TypeError, "cannot create value object from 'A' object"
        ):
            bt2.create_value(a)


def _create_const_value(value):
    class MySink(bt2._UserSinkComponent):
        def _user_consume(self):
            pass

        @classmethod
        def _user_query(cls, priv_query_exec, obj, params, method_obj):
            nonlocal value
            return {'my_value': value}

    res = bt2.QueryExecutor(MySink, 'obj', None).query()
    return res['my_value']


class BoolValueTestCase(_TestNumericValue, unittest.TestCase):
    def setUp(self):
        self._f = bt2.BoolValue(False)
        self._t = bt2.BoolValue(True)
        self._def = self._f
        self._def_value = False
        self._def_new_value = True

    def tearDown(self):
        del self._f
        del self._t
        del self._def

    def _assert_expecting_bool(self):
        return self.assertRaisesRegex(TypeError, r"expecting a 'bool' object")

    def test_create_default(self):
        b = bt2.BoolValue()
        self.assertFalse(b)

    def test_create_false(self):
        self.assertFalse(self._f)

    def test_create_true(self):
        self.assertTrue(self._t)

    def test_create_from_vfalse(self):
        b = bt2.BoolValue(self._f)
        self.assertFalse(b)

    def test_create_from_vtrue(self):
        b = bt2.BoolValue(self._t)
        self.assertTrue(b)

    def test_create_from_int_non_zero(self):
        with self.assertRaises(TypeError):
            bt2.BoolValue(23)

    def test_create_from_int_zero(self):
        with self.assertRaises(TypeError):
            bt2.BoolValue(0)

    def test_assign_true(self):
        b = bt2.BoolValue()
        b.value = True
        self.assertTrue(b)

    def test_assign_false(self):
        b = bt2.BoolValue()
        b.value = False
        self.assertFalse(b)

    def test_assign_vtrue(self):
        b = bt2.BoolValue()
        b.value = self._t
        self.assertTrue(b)

    def test_assign_vfalse(self):
        b = bt2.BoolValue()
        b.value = False
        self.assertFalse(b)

    def test_assign_int(self):
        with self.assertRaises(TypeError):
            b = bt2.BoolValue()
            b.value = 23

    def test_bool_op(self):
        self.assertEqual(bool(self._def), bool(self._def_value))

    def test_str_op(self):
        self.assertEqual(str(self._def), str(self._def_value))

    def test_eq_none(self):
        # Disable the "comparison to None" warning, as this is precisely what
        # we want to test here.
        self.assertFalse(self._def == None)  # noqa: E711

    def test_ne_none(self):
        # Disable the "comparison to None" warning, as this is precisely what
        # we want to test here.
        self.assertTrue(self._def != None)  # noqa: E711

    def test_vfalse_eq_false(self):
        self.assertEqual(self._f, False)

    def test_vfalse_ne_true(self):
        self.assertNotEqual(self._f, True)

    def test_vtrue_eq_true(self):
        self.assertEqual(self._t, True)

    def test_vtrue_ne_false(self):
        self.assertNotEqual(self._t, False)


_inject_numeric_testing_methods(BoolValueTestCase)


class _TestIntegerValue(_TestNumericValue):
    def setUp(self):
        self._pv = 23
        self._ip = self._CLS(self._pv)
        self._def = self._ip
        self._def_value = self._pv
        self._def_new_value = 101

    def tearDown(self):
        del self._ip
        del self._def
        del self._def_value

    def _assert_expecting_int(self):
        return self.assertRaisesRegex(TypeError, r'expecting an integral number object')

    def _assert_expecting_int64(self):
        return self.assertRaisesRegex(
            ValueError, r"expecting a signed 64-bit integral value"
        )

    def _assert_expecting_uint64(self):
        return self.assertRaisesRegex(
            ValueError, r"expecting an unsigned 64-bit integral value"
        )

    def test_create_default(self):
        i = self._CLS()
        self.assertEqual(i, 0)

    def test_create_pos(self):
        self.assertEqual(self._ip, self._pv)

    def test_create_neg(self):
        self.assertEqual(self._in, self._nv)

    def test_create_from_vint(self):
        i = self._CLS(self._ip)
        self.assertEqual(i, self._pv)

    def test_create_from_false(self):
        i = self._CLS(False)
        self.assertFalse(i)

    def test_create_from_true(self):
        i = self._CLS(True)
        self.assertTrue(i)

    def test_create_from_unknown(self):
        class A:
            pass

        with self._assert_expecting_int():
            self._CLS(A())

    def test_create_from_varray(self):
        with self._assert_expecting_int():
            self._CLS(bt2.ArrayValue())

    def test_assign_true(self):
        raw = True
        self._def.value = raw
        self.assertEqual(self._def, raw)

    def test_assign_false(self):
        raw = False
        self._def.value = raw
        self.assertEqual(self._def, raw)

    def test_assign_pos_int(self):
        raw = 477
        self._def.value = raw
        self.assertEqual(self._def, raw)

    def test_assign_vint(self):
        raw = 999
        self._def.value = bt2.create_value(raw)
        self.assertEqual(self._def, raw)


class SignedIntegerValueTestCase(_TestIntegerValue, unittest.TestCase):
    _CLS = bt2.SignedIntegerValue

    def setUp(self):
        super().setUp()
        self._nv = -52
        self._in = self._CLS(self._nv)
        self._def_new_value = -101

    def tearDown(self):
        super().tearDown()
        del self._in

    def test_create_neg(self):
        self.assertEqual(self._in, self._nv)

    def test_create_pos_too_big(self):
        with self._assert_expecting_int64():
            self._CLS(2 ** 63)

    def test_create_neg_too_big(self):
        with self._assert_expecting_int64():
            self._CLS(-(2 ** 63) - 1)

    def test_assign_neg_int(self):
        raw = -13
        self._def.value = raw
        self.assertEqual(self._def, raw)

    def test_compare_big_int(self):
        # Larger than the IEEE 754 double-precision exact representation of
        # integers.
        raw = (2 ** 53) + 1
        v = bt2.create_value(raw)
        self.assertEqual(v, raw)


_inject_numeric_testing_methods(SignedIntegerValueTestCase)


class UnsignedIntegerValueTestCase(_TestIntegerValue, unittest.TestCase):
    _CLS = bt2.UnsignedIntegerValue

    def test_create_pos_too_big(self):
        with self._assert_expecting_uint64():
            self._CLS(2 ** 64)

    def test_create_neg(self):
        with self._assert_expecting_uint64():
            self._CLS(-1)


_inject_numeric_testing_methods(UnsignedIntegerValueTestCase)


class RealValueTestCase(_TestNumericValue, unittest.TestCase):
    def setUp(self):
        self._pv = 23.4
        self._nv = -52.7
        self._fp = bt2.RealValue(self._pv)
        self._fn = bt2.RealValue(self._nv)
        self._def = self._fp
        self._def_value = self._pv
        self._def_new_value = -101.88

    def tearDown(self):
        del self._fp
        del self._fn
        del self._def
        del self._def_value

    def _assert_expecting_float(self):
        return self.assertRaisesRegex(TypeError, r"expecting a real number object")

    def _test_invalid_op(self, cb):
        with self.assertRaises(TypeError):
            cb()

    def test_create_default(self):
        f = bt2.RealValue()
        self.assertEqual(f, 0.0)

    def test_create_pos(self):
        self.assertEqual(self._fp, self._pv)

    def test_create_neg(self):
        self.assertEqual(self._fn, self._nv)

    def test_create_from_false(self):
        f = bt2.RealValue(False)
        self.assertFalse(f)

    def test_create_from_true(self):
        f = bt2.RealValue(True)
        self.assertTrue(f)

    def test_create_from_int(self):
        raw = 17
        f = bt2.RealValue(raw)
        self.assertEqual(f, float(raw))

    def test_create_from_vint(self):
        raw = 17
        f = bt2.RealValue(bt2.create_value(raw))
        self.assertEqual(f, float(raw))

    def test_create_from_vfloat(self):
        raw = 17.17
        f = bt2.RealValue(bt2.create_value(raw))
        self.assertEqual(f, raw)

    def test_create_from_unknown(self):
        class A:
            pass

        with self._assert_expecting_float():
            bt2.RealValue(A())

    def test_create_from_varray(self):
        with self._assert_expecting_float():
            bt2.RealValue(bt2.ArrayValue())

    def test_assign_true(self):
        self._def.value = True
        self.assertTrue(self._def)

    def test_assign_false(self):
        self._def.value = False
        self.assertFalse(self._def)

    def test_assign_pos_int(self):
        raw = 477
        self._def.value = raw
        self.assertEqual(self._def, float(raw))

    def test_assign_neg_int(self):
        raw = -13
        self._def.value = raw
        self.assertEqual(self._def, float(raw))

    def test_assign_vint(self):
        raw = 999
        self._def.value = bt2.create_value(raw)
        self.assertEqual(self._def, float(raw))

    def test_assign_float(self):
        raw = -19.23
        self._def.value = raw
        self.assertEqual(self._def, raw)

    def test_assign_vfloat(self):
        raw = 101.32
        self._def.value = bt2.create_value(raw)
        self.assertEqual(self._def, raw)

    def test_invalid_lshift(self):
        self._test_invalid_op(lambda: self._def << 23)

    def test_invalid_rshift(self):
        self._test_invalid_op(lambda: self._def >> 23)

    def test_invalid_and(self):
        self._test_invalid_op(lambda: self._def & 23)

    def test_invalid_or(self):
        self._test_invalid_op(lambda: self._def | 23)

    def test_invalid_xor(self):
        self._test_invalid_op(lambda: self._def ^ 23)

    def test_invalid_invert(self):
        self._test_invalid_op(lambda: ~self._def)


_inject_numeric_testing_methods(RealValueTestCase)


class StringValueTestCase(_TestCopySimple, unittest.TestCase):
    def setUp(self):
        self._def_value = 'Hello, World!'
        self._def = bt2.StringValue(self._def_value)
        self._def_const = _create_const_value(self._def_value)
        self._def_new_value = 'Yes!'

    def tearDown(self):
        del self._def

    def _assert_expecting_str(self):
        return self.assertRaises(TypeError)

    def test_create_default(self):
        s = bt2.StringValue()
        self.assertEqual(s, '')

    def test_create_from_str(self):
        raw = 'liberté'
        s = bt2.StringValue(raw)
        self.assertEqual(s, raw)

    def test_create_from_vstr(self):
        raw = 'liberté'
        s = bt2.StringValue(bt2.create_value(raw))
        self.assertEqual(s, raw)

    def test_create_from_unknown(self):
        class A:
            pass

        with self._assert_expecting_str():
            bt2.StringValue(A())

    def test_create_from_varray(self):
        with self._assert_expecting_str():
            bt2.StringValue(bt2.ArrayValue())

    def test_assign_int(self):
        with self._assert_expecting_str():
            self._def.value = 283

    def test_assign_str(self):
        raw = 'zorg'
        self._def = raw
        self.assertEqual(self._def, raw)

    def test_assign_vstr(self):
        raw = 'zorg'
        self._def = bt2.create_value(raw)
        self.assertEqual(self._def, raw)

    def test_eq(self):
        self.assertEqual(self._def, self._def_value)

    def test_const_eq(self):
        self.assertEqual(self._def_const, self._def_value)

    def test_eq_raw(self):
        self.assertNotEqual(self._def, 23)

    def test_lt_vstring(self):
        s1 = bt2.StringValue('allo')
        s2 = bt2.StringValue('bateau')
        self.assertLess(s1, s2)

    def test_lt_string(self):
        s1 = bt2.StringValue('allo')
        self.assertLess(s1, 'bateau')

    def test_le_vstring(self):
        s1 = bt2.StringValue('allo')
        s2 = bt2.StringValue('bateau')
        self.assertLessEqual(s1, s2)

    def test_le_string(self):
        s1 = bt2.StringValue('allo')
        self.assertLessEqual(s1, 'bateau')

    def test_gt_vstring(self):
        s1 = bt2.StringValue('allo')
        s2 = bt2.StringValue('bateau')
        self.assertGreater(s2, s1)

    def test_gt_string(self):
        s1 = bt2.StringValue('allo')
        self.assertGreater('bateau', s1)

    def test_ge_vstring(self):
        s1 = bt2.StringValue('allo')
        s2 = bt2.StringValue('bateau')
        self.assertGreaterEqual(s2, s1)

    def test_ge_string(self):
        s1 = bt2.StringValue('allo')
        self.assertGreaterEqual('bateau', s1)

    def test_in_string(self):
        s1 = bt2.StringValue('beau grand bateau')
        self.assertIn('bateau', s1)

    def test_in_vstring(self):
        s1 = bt2.StringValue('beau grand bateau')
        s2 = bt2.StringValue('bateau')
        self.assertIn(s2, s1)

    def test_bool_op(self):
        self.assertEqual(bool(self._def), bool(self._def_value))

    def test_str_op(self):
        self.assertEqual(str(self._def), str(self._def_value))

    def test_len(self):
        self.assertEqual(len(self._def), len(self._def_value))

    def test_getitem(self):
        self.assertEqual(self._def[5], self._def_value[5])

    def test_const_getitem(self):
        self.assertEqual(self._def_const[5], self._def_value[5])

    def test_iadd_str(self):
        to_append = 'meow meow meow'
        self._def += to_append
        self._def_value += to_append
        self.assertEqual(self._def, self._def_value)

    def test_const_iadd_str(self):
        to_append = 'meow meow meow'
        with self.assertRaises(TypeError):
            self._def_const += to_append

        self.assertEqual(self._def_const, self._def_value)

    def test_append_vstr(self):
        to_append = 'meow meow meow'
        self._def += bt2.create_value(to_append)
        self._def_value += to_append
        self.assertEqual(self._def, self._def_value)


class ArrayValueTestCase(_TestCopySimple, unittest.TestCase):
    def setUp(self):
        self._def_value = [None, False, True, -23, 0, 42, -42.4, 23.17, 'yes']
        self._def = bt2.ArrayValue(copy.deepcopy(self._def_value))
        self._def_const = _create_const_value(copy.deepcopy(self._def_value))

    def tearDown(self):
        del self._def

    def _modify_def(self):
        self._def[2] = 'xyz'

    def _assert_type_error(self):
        return self.assertRaises(TypeError)

    def test_create_default(self):
        a = bt2.ArrayValue()
        self.assertEqual(len(a), 0)

    def test_create_from_array(self):
        self.assertEqual(self._def, self._def_value)

    def test_create_from_tuple(self):
        t = 1, 2, False, None
        a = bt2.ArrayValue(t)
        self.assertEqual(a, t)

    def test_create_from_varray(self):
        va = bt2.ArrayValue(copy.deepcopy(self._def_value))
        a = bt2.ArrayValue(va)
        self.assertEqual(va, a)

    def test_create_from_unknown(self):
        class A:
            pass

        with self._assert_type_error():
            bt2.ArrayValue(A())

    def test_bool_op_true(self):
        self.assertTrue(bool(self._def))

    def test_bool_op_false(self):
        self.assertFalse(bool(bt2.ArrayValue()))

    def test_len(self):
        self.assertEqual(len(self._def), len(self._def_value))

    def test_eq_int(self):
        self.assertNotEqual(self._def, 23)

    def test_const_eq(self):
        a1 = _create_const_value([1, 2, 3])
        a2 = [1, 2, 3]
        self.assertEqual(a1, a2)

    def test_eq_diff_len(self):
        a1 = bt2.create_value([1, 2, 3])
        a2 = bt2.create_value([1, 2])
        self.assertIs(type(a1), bt2.ArrayValue)
        self.assertIs(type(a2), bt2.ArrayValue)
        self.assertNotEqual(a1, a2)

    def test_eq_diff_content_same_len(self):
        a1 = bt2.create_value([1, 2, 3])
        a2 = bt2.create_value([4, 5, 6])
        self.assertNotEqual(a1, a2)

    def test_eq_same_content_same_len(self):
        raw = (3, True, [1, 2.5, None, {'a': 17.6, 'b': None}])
        a1 = bt2.ArrayValue(raw)
        a2 = bt2.ArrayValue(copy.deepcopy(raw))
        self.assertEqual(a1, a2)

    def test_eq_non_sequence_iterable(self):
        dct = collections.OrderedDict([(1, 2), (3, 4), (5, 6)])
        a = bt2.ArrayValue((1, 3, 5))
        self.assertEqual(a, list(dct.keys()))
        self.assertNotEqual(a, dct)

    def test_setitem_int(self):
        raw = 19
        self._def[2] = raw
        self.assertEqual(self._def[2], raw)

    def test_setitem_vint(self):
        raw = 19
        self._def[2] = bt2.create_value(raw)
        self.assertEqual(self._def[2], raw)

    def test_setitem_none(self):
        self._def[2] = None
        self.assertIsNone(self._def[2])

    def test_setitem_index_wrong_type(self):
        with self._assert_type_error():
            self._def['yes'] = 23

    def test_setitem_index_neg(self):
        with self.assertRaises(IndexError):
            self._def[-2] = 23

    def test_setitem_index_out_of_range(self):
        with self.assertRaises(IndexError):
            self._def[len(self._def)] = 23

    def test_const_setitem(self):
        with self.assertRaises(TypeError):
            self._def_const[2] = 19

    def test_append_none(self):
        self._def.append(None)
        self.assertIsNone(self._def[len(self._def) - 1])

    def test_append_int(self):
        raw = 145
        self._def.append(raw)
        self.assertEqual(self._def[len(self._def) - 1], raw)

    def test_const_append(self):
        with self.assertRaises(AttributeError):
            self._def_const.append(12194)

    def test_append_vint(self):
        raw = 145
        self._def.append(bt2.create_value(raw))
        self.assertEqual(self._def[len(self._def) - 1], raw)

    def test_append_unknown(self):
        class A:
            pass

        with self._assert_type_error():
            self._def.append(A())

    def test_iadd(self):
        raw = 4, 5, True
        self._def += raw
        self.assertEqual(self._def[len(self._def) - 3], raw[0])
        self.assertEqual(self._def[len(self._def) - 2], raw[1])
        self.assertEqual(self._def[len(self._def) - 1], raw[2])

    def test_const_iadd(self):
        with self.assertRaises(TypeError):
            self._def_const += 12194

    def test_iadd_unknown(self):
        class A:
            pass

        with self._assert_type_error():
            self._def += A()

    def test_iadd_list_unknown(self):
        class A:
            pass

        with self._assert_type_error():
            self._def += [A()]

    def test_iter(self):
        for velem, elem in zip(self._def, self._def_value):
            self.assertEqual(velem, elem)

    def test_const_iter(self):
        for velem, elem in zip(self._def_const, self._def_value):
            self.assertEqual(velem, elem)

    def test_const_get_item(self):
        item1 = self._def_const[0]
        item2 = self._def_const[2]
        item3 = self._def_const[5]
        item4 = self._def_const[7]
        item5 = self._def_const[8]

        self.assertEqual(item1, None)

        self.assertIs(type(item2), bt2._BoolValueConst)
        self.assertEqual(item2, True)

        self.assertIs(type(item3), bt2._SignedIntegerValueConst)
        self.assertEqual(item3, 42)

        self.assertIs(type(item4), bt2._RealValueConst)
        self.assertEqual(item4, 23.17)

        self.assertIs(type(item5), bt2._StringValueConst)
        self.assertEqual(item5, 'yes')


class MapValueTestCase(_TestCopySimple, unittest.TestCase):
    def setUp(self):
        self._def_value = {
            'none': None,
            'false': False,
            'true': True,
            'neg-int': -23,
            'zero': 0,
            'pos-int': 42,
            'neg-float': -42.4,
            'pos-float': 23.17,
            'str': 'yes',
        }
        self._def = bt2.MapValue(copy.deepcopy(self._def_value))
        self._def_const = _create_const_value(self._def_value)

    def tearDown(self):
        del self._def

    def _modify_def(self):
        self._def['zero'] = 1

    def test_create_default(self):
        m = bt2.MapValue()
        self.assertEqual(len(m), 0)

    def test_create_from_dict(self):
        self.assertEqual(self._def, self._def_value)

    def test_create_from_vmap(self):
        vm = bt2.MapValue(copy.deepcopy(self._def_value))
        m = bt2.MapValue(vm)
        self.assertEqual(vm, m)

    def test_create_from_unknown(self):
        class A:
            pass

        with self.assertRaises(AttributeError):
            bt2.MapValue(A())

    def test_bool_op_true(self):
        self.assertTrue(bool(self._def))

    def test_bool_op_false(self):
        self.assertFalse(bool(bt2.MapValue()))

    def test_len(self):
        self.assertEqual(len(self._def), len(self._def_value))

    def test_const_eq(self):
        a1 = _create_const_value({'a': 1, 'b': 2, 'c': 3})
        a2 = {'a': 1, 'b': 2, 'c': 3}
        self.assertEqual(a1, a2)

    def test_eq_int(self):
        self.assertNotEqual(self._def, 23)

    def test_eq_diff_len(self):
        a1 = bt2.create_value({'a': 1, 'b': 2, 'c': 3})
        a2 = bt2.create_value({'a': 1, 'b': 2})
        self.assertNotEqual(a1, a2)

    def test_const_eq_diff_len(self):
        a1 = _create_const_value({'a': 1, 'b': 2, 'c': 3})
        a2 = _create_const_value({'a': 1, 'b': 2})
        self.assertNotEqual(a1, a2)

    def test_eq_diff_content_same_len(self):
        a1 = bt2.create_value({'a': 1, 'b': 2, 'c': 3})
        a2 = bt2.create_value({'a': 4, 'b': 2, 'c': 3})
        self.assertNotEqual(a1, a2)

    def test_const_eq_diff_content_same_len(self):
        a1 = _create_const_value({'a': 1, 'b': 2, 'c': 3})
        a2 = _create_const_value({'a': 4, 'b': 2, 'c': 3})
        self.assertNotEqual(a1, a2)

    def test_eq_same_content_diff_keys(self):
        a1 = bt2.create_value({'a': 1, 'b': 2, 'c': 3})
        a2 = bt2.create_value({'a': 1, 'k': 2, 'c': 3})
        self.assertNotEqual(a1, a2)

    def test_const_eq_same_content_diff_keys(self):
        a1 = _create_const_value({'a': 1, 'b': 2, 'c': 3})
        a2 = _create_const_value({'a': 1, 'k': 2, 'c': 3})
        self.assertNotEqual(a1, a2)

    def test_eq_same_content_same_len(self):
        raw = {'3': 3, 'True': True, 'array': [1, 2.5, None, {'a': 17.6, 'b': None}]}
        a1 = bt2.MapValue(raw)
        a2 = bt2.MapValue(copy.deepcopy(raw))
        self.assertEqual(a1, a2)
        self.assertEqual(a1, raw)

    def test_const_eq_same_content_same_len(self):
        raw = {'3': 3, 'True': True, 'array': [1, 2.5, None, {'a': 17.6, 'b': None}]}
        a1 = _create_const_value(raw)
        a2 = _create_const_value(copy.deepcopy(raw))
        self.assertEqual(a1, a2)
        self.assertEqual(a1, raw)

    def test_setitem_int(self):
        raw = 19
        self._def['pos-int'] = raw
        self.assertEqual(self._def['pos-int'], raw)

    def test_const_setitem_int(self):
        with self.assertRaises(TypeError):
            self._def_const['pos-int'] = 19

    def test_setitem_vint(self):
        raw = 19
        self._def['pos-int'] = bt2.create_value(raw)
        self.assertEqual(self._def['pos-int'], raw)

    def test_setitem_none(self):
        self._def['none'] = None
        self.assertIsNone(self._def['none'])

    def test_setitem_new_int(self):
        old_len = len(self._def)
        self._def['new-int'] = 23
        self.assertEqual(self._def['new-int'], 23)
        self.assertEqual(len(self._def), old_len + 1)

    def test_setitem_index_wrong_type(self):
        with self.assertRaises(TypeError):
            self._def[18] = 23

    def test_iter(self):
        for vkey, vval in self._def.items():
            val = self._def_value[vkey]
            self.assertEqual(vval, val)

    def test_const_iter(self):
        for vkey, vval in self._def_const.items():
            val = self._def_value[vkey]
            self.assertEqual(vval, val)

    def test_get_item(self):
        i = self._def['pos-float']
        self.assertIs(type(i), bt2.RealValue)
        self.assertEqual(i, 23.17)

    def test_const_get_item(self):
        item1 = self._def_const['none']
        item2 = self._def_const['true']
        item3 = self._def_const['pos-int']
        item4 = self._def_const['pos-float']
        item5 = self._def_const['str']

        self.assertEqual(item1, None)

        self.assertIs(type(item2), bt2._BoolValueConst)
        self.assertEqual(item2, True)

        self.assertIs(type(item3), bt2._SignedIntegerValueConst)
        self.assertEqual(item3, 42)

        self.assertIs(type(item4), bt2._RealValueConst)
        self.assertEqual(item4, 23.17)

        self.assertIs(type(item5), bt2._StringValueConst)
        self.assertEqual(item5, 'yes')

    def test_getitem_wrong_key(self):
        with self.assertRaises(KeyError):
            self._def['kilojoule']


if __name__ == '__main__':
    unittest.main()



import bt2
import signal
import os
import time

bt2.register_plugin(__name__, "test_exit_status")


class StatusIter(bt2._UserMessageIterator):
    def __init__(self, config, output_port):
        self.case = output_port.user_data['case']

    def __next__(self):
        if self.case == "STOP":
            raise bt2.Stop()
        if self.case == "INTERRUPTED":
            os.kill(os.getpid(), signal.SIGINT)

            # Wait until the graph is in the interrupted state.
            timeout_s = 10
            for _ in range(timeout_s * 10):
                if self._is_interrupted:
                    raise bt2.TryAgain()

                time.sleep(0.1)

            raise Exception(
                '{} was not interrupted after {} seconds'.format(
                    self.__class__.__name__, timeout_s
                )
            )

        elif self.case == "ERROR":
            raise TypeError("Raising type error")
        else:
            raise ValueError("Invalid parameter")


@bt2.plugin_component_class
class StatusSrc(bt2._UserSourceComponent, message_iterator_class=StatusIter):
    def __init__(self, config, params, obj):
        self._add_output_port("out", {'case': params['case']})



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import unittest
import bt2


class _DummySink(bt2._UserSinkComponent):
    def _user_consume(self):
        pass


class ComponentDescriptorTestCase(unittest.TestCase):
    def setUp(self):
        self._obj = object()
        self._comp_descr = bt2.ComponentDescriptor(_DummySink, {'zoom': -23}, self._obj)

    def _get_comp_cls_from_plugin(self):
        plugin = bt2.find_plugin('text', find_in_user_dir=False, find_in_sys_dir=False)
        assert plugin is not None
        cc = plugin.source_component_classes['dmesg']
        assert cc is not None
        return cc

    def test_init_invalid_cls_type(self):
        with self.assertRaises(TypeError):
            bt2.ComponentDescriptor(int)

    def test_init_invalid_params_type(self):
        with self.assertRaises(TypeError):
            bt2.ComponentDescriptor(_DummySink, object())

    def test_init_invalid_obj_non_python_comp_cls(self):
        cc = self._get_comp_cls_from_plugin()

        with self.assertRaises(ValueError):
            bt2.ComponentDescriptor(cc, obj=57)

    def test_init_with_user_comp_cls(self):
        bt2.ComponentDescriptor(_DummySink)

    def test_init_with_gen_comp_cls(self):
        cc = self._get_comp_cls_from_plugin()
        bt2.ComponentDescriptor(cc)

    def test_attr_component_class(self):
        self.assertIs(self._comp_descr.component_class, _DummySink)

    def test_attr_params(self):
        self.assertEqual(self._comp_descr.params, {'zoom': -23})

    def test_attr_obj(self):
        self.assertIs(self._comp_descr.obj, self._obj)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2016, Matt Layman

from __future__ import print_function
import os
import string
import sys

from tap.directive import Directive
from tap.i18n import _
from tap.line import Result


class Tracker(object):

    def __init__(
            self, outdir=None, combined=False, streaming=False, stream=None,
            header=True):
        self.outdir = outdir

        # Combine all the test results into one file.
        self.combined = combined
        self.combined_line_number = 0
        # Test case ordering is important for the combined results
        # because of how numbers are assigned. The test cases
        # must be tracked in order so that reporting can sequence
        # the line numbers properly.
        self.combined_test_cases_seen = []

        # Stream output directly to a stream instead of file output.
        self.streaming = streaming
        self.stream = stream

        # Display the test case header unless told not to.
        self.header = header

        # Internal state for tracking each test case.
        self._test_cases = {}

        # Python versions 2 and 3 keep maketrans in different locations.
        if sys.version_info[0] < 3:
            self._sanitized_table = string.maketrans(' \\/\n', '----')
        else:  # pragma: no cover
            self._sanitized_table = str.maketrans(' \\/\n', '----')

    def _get_outdir(self):
        return self._outdir

    def _set_outdir(self, outdir):
        self._outdir = outdir
        if outdir and not os.path.exists(outdir):
            os.makedirs(outdir)

    outdir = property(_get_outdir, _set_outdir)

    def _track(self, class_name):
        """Keep track of which test cases have executed."""
        if self._test_cases.get(class_name) is None:
            if self.streaming and self.header:
                self._write_test_case_header(class_name, self.stream)

            self._test_cases[class_name] = []
            if self.combined:
                self.combined_test_cases_seen.append(class_name)

    def add_ok(self, class_name, description, directive=''):
        result = Result(
            ok=True, number=self._get_next_line_number(class_name),
            description=description, directive=Directive(directive))
        self._add_line(class_name, result)

    def add_not_ok(
            self, class_name, description, directive='', diagnostics=None):
        result = Result(
            ok=False, number=self._get_next_line_number(class_name),
            description=description, diagnostics=diagnostics,
            directive=Directive(directive))
        self._add_line(class_name, result)

    def add_skip(self, class_name, description, reason):
        directive = 'SKIP {0}'.format(reason)
        result = Result(
            ok=True, number=self._get_next_line_number(class_name),
            description=description, directive=Directive(directive))
        self._add_line(class_name, result)

    def _add_line(self, class_name, result):
        self._track(class_name)
        if self.streaming:
            print(result, file=self.stream)
        self._test_cases[class_name].append(result)

    def _get_next_line_number(self, class_name):
        if self.combined or self.streaming:
            # This has an obvious side effect. Oh well.
            self.combined_line_number += 1
            return self.combined_line_number
        else:
            try:
                return len(self._test_cases[class_name]) + 1
            except KeyError:
                # A result is created before the call to _track so the test
                # case may not be tracked yet. In that case, the line is 1.
                return 1

    def generate_tap_reports(self):
        """Generate TAP reports.

        The results are either combined into a single output file or
        the output file name is generated from the test case.
        """
        if self.streaming:
            # The results already went to the stream, record the plan.
            print('1..{0}'.format(self.combined_line_number), file=self.stream)
            return

        if self.combined:
            combined_file = 'testresults.tap'
            if self.outdir:
                combined_file = os.path.join(self.outdir, combined_file)
            with open(combined_file, 'w') as out_file:
                for test_case in self.combined_test_cases_seen:
                    self.generate_tap_report(
                        test_case, self._test_cases[test_case], out_file)
                print(
                    '1..{0}'.format(self.combined_line_number), file=out_file)
        else:
            for test_case, tap_lines in self._test_cases.items():
                with open(self._get_tap_file_path(test_case), 'w') as out_file:
                    self.generate_tap_report(test_case, tap_lines, out_file)

    def generate_tap_report(self, test_case, tap_lines, out_file):
        self._write_test_case_header(test_case, out_file)

        for tap_line in tap_lines:
            print(tap_line, file=out_file)

        # For combined results, the plan is only output once after
        # all the test cases complete.
        if not self.combined:
            print('1..{0}'.format(len(tap_lines)), file=out_file)

    def _write_test_case_header(self, test_case, stream):
        print(_('# TAP results for {test_case}').format(
            test_case=test_case), file=stream)

    def _get_tap_file_path(self, test_case):
        """Get the TAP output file path for the test case."""
        sanitized_test_case = test_case.translate(self._sanitized_table)
        tap_file = sanitized_test_case + '.tap'
        if self.outdir:
            return os.path.join(self.outdir, tap_file)
        return tap_file
